@@ -4,8 +4,8 @@
  *
  */
 
-#ifndef _COMMON_H
-#define _COMMON_H
+#ifndef GUARD_COMMON_H
+#define GUARD_COMMON_H
 
 #ifndef __attribute__
 # if !defined __GNUC__ || __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 8) || __STRICT_ANSI__
@@ -19,7 +19,7 @@
 extern char *progname;
 
 /* Finish programs being */
-void die(int, char *, ...) __attribute__ ((__format__ (__printf__, 2, 3)));
+void __attribute ((noreturn)) die(int, char *, ...) __attribute__ ((__format__ (__printf__, 2, 3)));
 
 /* Print an error */
 void errstr(char *, ...) __attribute__ ((__format__ (__printf__, 1, 2)));
@@ -46,4 +46,4 @@ char *sstrdup(const char *s);
 /* Print version string */
 void version(void);
 
-#endif /* _COMMON_H */
+#endif /* GUARD_COMMON_H */
