@@ -32,9 +32,7 @@
  * SUCH DAMAGE.
  */
 
-#ident "$Copyright: (c) 1980, 1990 Regents of the University of California. $"
-#ident "$Copyright: All rights reserved. $"
-#ident "$Id: edquota.c,v 1.22 2007/09/18 16:21:07 jkar8572 Exp $"
+#include "config.h"
 
 /*
  * Disk quota editor.
@@ -68,12 +66,12 @@
 
 char *progname;
 
-int flags, quotatype;
-int fmt = -1;
-char *protoname;
-char *dirname;
+static int flags, quotatype;
+static int fmt = -1;
+static char *protoname;
+static char *dirname;
 
-void usage(void)
+static void usage(void)
 {
 #if defined(RPC_SETQUOTA)
 	char *rpcflag = "[-rm] ";
@@ -82,8 +80,8 @@ void usage(void)
 #endif
 	errstr(_("Usage:\n\tedquota %1$s[-u] [-F formatname] [-p username] [-f filesystem] username ...\n\
 \tedquota %1$s-g [-F formatname] [-p groupname] [-f filesystem] groupname ...\n\
-\tedquota %1$s[-u|g] [-F formatname] [-f filesystem] -t\n\
-\tedquota %1$s[-u|g] [-F formatname] [-f filesystem] -T username|groupname ...\n"), rpcflag);
+\tedquota [-u|g] [-F formatname] [-f filesystem] -t\n\
+\tedquota [-u|g] [-F formatname] [-f filesystem] -T username|groupname ...\n"), rpcflag);
 	fputs(_("\n\
 -u, --user                    edit user data\n\
 -g, --group                   edit group data\n"), stderr);
@@ -104,7 +102,7 @@ void usage(void)
 	exit(1);
 }
 
-int parse_options(int argc, char **argv)
+static int parse_options(int argc, char **argv)
 {
 	int ret;
 	struct option long_opts[] = {
@@ -130,9 +128,9 @@ int parse_options(int argc, char **argv)
 
 	quotatype = USRQUOTA;
 #if defined(RPC_SETQUOTA)
-	while ((ret = getopt_long(argc, argv, "ugrmntTVp:F:f:", long_opts, NULL)) != -1) {
+	while ((ret = getopt_long(argc, argv, "ughrmntTVp:F:f:", long_opts, NULL)) != -1) {
 #else
-	while ((ret = getopt_long(argc, argv, "ugtTVp:F:f:", long_opts, NULL)) != -1) {
+	while ((ret = getopt_long(argc, argv, "ughtTVp:F:f:", long_opts, NULL)) != -1) {
 #endif
 		switch (ret) {
 		  case 'p':
@@ -172,6 +170,7 @@ int parse_options(int argc, char **argv)
 		  case 'V':
 			  version();
 			  exit(0);
+		  case 'h':
 		  default:
 			  usage();
 		}
@@ -181,13 +180,17 @@ int parse_options(int argc, char **argv)
 	if (((flags & FL_EDIT_PERIOD) && argc != 0) || ((flags & FL_EDIT_TIMES) && argc < 1))
 		usage();
 	if ((flags & (FL_EDIT_PERIOD | FL_EDIT_TIMES)) && protoname) {
-		errstr(_("Prototype name does not make sence when editting grace period or times.\n"));
+		errstr(_("Prototype name does not make sense when editing grace period or times.\n"));
+		usage();
+	}
+	if (flags & FL_REMOTE && (flags & (FL_EDIT_TIMES | FL_EDIT_PERIOD))) {
+		errstr(_("Cannot change grace times over RPC protocol.\n"));
 		usage();
 	}
 	return optind;
 }
 
-void copy_prototype(int argc, char **argv, struct quota_handle **handles)
+static void copy_prototype(int argc, char **argv, struct quota_handle **handles)
 {
 	int ret, protoid, id;
 	struct dquot *protoprivs, *curprivs, *pprivs, *cprivs;
@@ -196,8 +199,11 @@ void copy_prototype(int argc, char **argv, struct quota_handle **handles)
 	protoid = name2id(protoname, quotatype, !!(flags & FL_NUMNAMES), NULL);
 	protoprivs = getprivs(protoid, handles, 0);
 	while (argc-- > 0) {
-		id = name2id(*argv++, quotatype, !!(flags & FL_NUMNAMES), NULL);
+		id = name2id(*argv, quotatype, !!(flags & FL_NUMNAMES), NULL);
 		curprivs = getprivs(id, handles, 0);
+		if (!curprivs)
+			die(1, _("Cannot get quota information for user %s\n"), *argv);
+		argv++;
 
 		for (pprivs = protoprivs, cprivs = curprivs; pprivs && cprivs;
 		     pprivs = pprivs->dq_next, cprivs = cprivs->dq_next) {
@@ -276,7 +282,7 @@ int main(int argc, char **argv)
 			ret = -1;
 		}
 		if (editprivs(tmpfil) < 0) {
-			errstr(_("Error while editting grace times.\n"));
+			errstr(_("Error while editing grace times.\n"));
 			ret = -1;
 		}
 		if (readtimes(handles, tmpfd) < 0) {
@@ -288,13 +294,15 @@ int main(int argc, char **argv)
 		for (; argc > 0; argc--, argv++) {
 			id = name2id(*argv, quotatype, !!(flags & FL_NUMNAMES), NULL);
 			curprivs = getprivs(id, handles, 0);
+			if (!curprivs)
+				die(1, _("Cannot get quota information for user %s.\n"), *argv);
 			if (writeindividualtimes(curprivs, tmpfd, *argv, quotatype) < 0) {
 				errstr(_("Cannot write individual grace times to file.\n"));
 				ret = -1;
 				continue;
 			}
 			if (editprivs(tmpfil) < 0) {
-				errstr(_("Error while editting individual grace times.\n"));
+				errstr(_("Error while editing individual grace times.\n"));
 				ret = -1;
 				continue;
 			}
@@ -312,13 +320,15 @@ int main(int argc, char **argv)
 		for (; argc > 0; argc--, argv++) {
 			id = name2id(*argv, quotatype, !!(flags & FL_NUMNAMES), NULL);
 			curprivs = getprivs(id, handles, 0);
+			if (!curprivs)
+				die(1, _("Cannot get quota information for user %s.\n"), *argv);
 			if (writeprivs(curprivs, tmpfd, *argv, quotatype) < 0) {
 				errstr(_("Cannot write quotas to file.\n"));
 				ret = -1;
 				continue;
 			}
 			if (editprivs(tmpfil) < 0) {
-				errstr(_("Error while editting quotas.\n"));
+				errstr(_("Error while editing quotas.\n"));
 				ret = -1;
 				continue;
 			}
