@@ -32,9 +32,7 @@
  * SUCH DAMAGE.
  */
 
-#ident "$Copyright: (c) 1980, 1990 Regents of the University of California. $"
-#ident "$Copyright: (c) 2000, 2001 Silicon Graphics, Inc. $"
-#ident "$Copyright: All rights reserved. $"
+#include "config.h"
 
 #include <sys/stat.h>
 #include <sys/param.h>
@@ -58,8 +56,8 @@
 #include "quotasys.h"
 
 #define	TSIZE	500
-__uint64_t sizes[TSIZE];
-__uint64_t overflow;
+static __uint64_t sizes[TSIZE];
+static __uint64_t overflow;
 
 static int aflag;
 static int cflag;
