@@ -1,10 +1,10 @@
-#ifndef _QUOTA_H
-#define _QUOTA_H
+#ifndef GUARD_QUOTA_H
+#define GUARD_QUOTA_H
 
 #include <sys/types.h>
 
 typedef u_int32_t qid_t;	/* Type in which we store ids in memory */
-typedef u_int64_t qsize_t;	/* Type in which we store size limitations */
+typedef int64_t qsize_t;	/* Type in which we store size limitations */
 
 #define MAXQUOTAS 2
 #define USRQUOTA  0		/* element used for user quotas */
@@ -112,13 +112,17 @@ struct if_dqinfo {
 #define QUOTA_NL_BHARDWARN 4            /* Block hardlimit reached */
 #define QUOTA_NL_BSOFTLONGWARN 5        /* Block grace time expired */
 #define QUOTA_NL_BSOFTWARN 6            /* Block softlimit reached */
+#define QUOTA_NL_IHARDBELOW 7		/* Usage got below inode hardlimit */
+#define QUOTA_NL_ISOFTBELOW 8		/* Usage got below inode softlimit */
+#define QUOTA_NL_BHARDBELOW 9		/* Usage got below block hardlimit */
+#define QUOTA_NL_BSOFTBELOW 10		/* Usage got below block softlimit */
 
 enum {
 	QUOTA_NL_C_UNSPEC,
 	QUOTA_NL_C_WARNING,
-	__QUOTA_NL_C_MAX,
+	ENUM_QUOTA_NL_C_MAX,
 };
-#define QUOTA_NL_C_MAX (__QUOTA_NL_C_MAX - 1)
+#define QUOTA_NL_C_MAX (ENUM_QUOTA_NL_C_MAX - 1)
 
 enum {
 	QUOTA_NL_A_UNSPEC,
@@ -128,13 +132,15 @@ enum {
 	QUOTA_NL_A_DEV_MAJOR,
 	QUOTA_NL_A_DEV_MINOR,
 	QUOTA_NL_A_CAUSED_ID,
-	__QUOTA_NL_A_MAX,
+	ENUM_QUOTA_NL_A_MAX,
 };
-#define QUOTA_NL_A_MAX (__QUOTA_NL_A_MAX - 1)
+#define QUOTA_NL_A_MAX (ENUM_QUOTA_NL_A_MAX - 1)
 
 /* Quota format identifiers */
 #define QFMT_VFS_OLD 1
 #define QFMT_VFS_V0  2
+#define QFMT_OCFS2   3
+#define QFMT_VFS_V1  4
 
 /* Flags supported by kernel */
 #define V1_DQF_RSQUASH 1
