@@ -15,6 +15,8 @@
  *
  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <unistd.h>
 #include <getopt.h>
@@ -22,6 +24,7 @@
 #include <errno.h>
 #include <string.h>
 #include <fcntl.h>
+#include <limits.h>
 
 #include <netlink/genl/genl.h>
 #include <netlink/genl/ctrl.h>
@@ -35,15 +38,6 @@
 
 char *progname;
 
-static const struct option options[] = {
-	{ "version", 0, NULL, 'V' },
-	{ "help", 0, NULL, 'h' },
-	{ "no-dbus", 0, NULL, 'D' },
-	{ "no-console", 0, NULL, 'C' },
-	{ "no-daemon", 0, NULL, 'F' },
-	{ NULL, 0, NULL, 0 }
-};
-
 struct quota_warning {
 	uint32_t qtype;
 	uint64_t excess_id;
@@ -66,16 +60,28 @@ static struct nla_policy quota_nl_warn_cmd_policy[QUOTA_NL_A_MAX+1] = {
 #define FL_NODBUS 1
 #define FL_NOCONSOLE 2
 #define FL_NODAEMON 4
+#define FL_PRINTBELOW 8
 
-int flags;
-DBusConnection *dhandle;
+static int flags;
+static DBusConnection *dhandle;
 
-void show_help(void)
+static const struct option options[] = {
+	{ "version", 0, NULL, 'V' },
+	{ "help", 0, NULL, 'h' },
+	{ "no-dbus", 0, NULL, 'D' },
+	{ "no-console", 0, NULL, 'C' },
+	{ "foreground", 0, NULL, 'F' },
+	{ "print-below", 0, NULL, 'b' },
+	{ NULL, 0, NULL, 0 }
+};
+
+static void show_help(void)
 {
 	errstr(_("Usage: %s [options]\nOptions are:\n\
  -h --help         shows this text\n\
  -V --version      shows version information\n\
  -C --no-console   do not try to write messages to console\n\
+ -b --print-below  write to console also information about getting below hard/soft limits\n\
  -D --no-dbus      do not try to write messages to DBUS\n\
  -F --foreground   run daemon in foreground\n"), progname);
 }
@@ -84,7 +90,7 @@ static void parse_options(int argc, char **argv)
 {
 	int opt;
 
-	while ((opt = getopt_long(argc, argv, "VhDCF", options, NULL)) >= 0) {
+	while ((opt = getopt_long(argc, argv, "VhDCFb", options, NULL)) >= 0) {
 		switch (opt) {
 			case 'V':
 				version();
@@ -101,6 +107,9 @@ static void parse_options(int argc, char **argv)
 			case 'F':
 				flags |= FL_NODAEMON;
 				break;
+			case 'b':
+				flags |= FL_PRINTBELOW;
+				break;
 			default:
 				errstr(_("Unknown option '%c'.\n"), opt);
 				show_help();
@@ -230,6 +239,11 @@ static void write_console_warning(struct quota_warning *warn)
 	char warnbuf[WARN_BUF_SIZE];
 	char *level, *msg;
 
+	if ((warn->warntype == QUOTA_NL_IHARDBELOW ||
+	    warn->warntype == QUOTA_NL_ISOFTBELOW ||
+	    warn->warntype == QUOTA_NL_BHARDBELOW ||
+	    warn->warntype == QUOTA_NL_BSOFTBELOW) && !(flags & FL_PRINTBELOW))
+		return;
 	uid2user(warn->caused_id, user);
 	strcpy(dev, "/dev/");
 
@@ -250,44 +264,60 @@ static void write_console_warning(struct quota_warning *warn)
 		}
 	}
 	if (!max_atime) {
-		errstr(_("Failed to find tty of user %Lu to report warning to.\n"), (unsigned long long)warn->caused_id);
+		errstr(_("Failed to find tty of user %llu to report warning to.\n"), (unsigned long long)warn->caused_id);
 		return;
 	}
 	fd = open(max_dev, O_WRONLY);
 	if (fd < 0) {
-		errstr(_("Failed to open tty %s of user %Lu to report warning.\n"), dev, (unsigned long long)warn->caused_id);
+		errstr(_("Failed to open tty %s of user %llu to report warning.\n"), dev, (unsigned long long)warn->caused_id);
 		return;
 	}
 	id2name(warn->excess_id, warn->qtype, user);
-	if (warn->warntype == QUOTA_NL_ISOFTWARN || warn->warntype == QUOTA_NL_BSOFTWARN)
-		level = "Warning";
+	if (warn->warntype == QUOTA_NL_ISOFTWARN ||
+	    warn->warntype == QUOTA_NL_BSOFTWARN)
+		level = _("Warning");
+	else if (warn->warntype == QUOTA_NL_IHARDWARN ||
+		 warn->warntype == QUOTA_NL_BHARDWARN)
+		level = _("Error");
 	else
-		level = "Error";
+		level = _("Info");
 	switch (warn->warntype) {
 		case QUOTA_NL_IHARDWARN:
-			msg = "file limit reached";
+			msg = _("file limit reached");
 			break;
 		case QUOTA_NL_ISOFTLONGWARN:
-			msg = "file quota exceeded too long";
+			msg = _("file quota exceeded too long");
 			break;
 		case QUOTA_NL_ISOFTWARN:
-			msg = "file quota exceeded";
+			msg = _("file quota exceeded");
 			break;
 		case QUOTA_NL_BHARDWARN:
-			msg = "block limit reached";
+			msg = _("block limit reached");
 			break;
 		case QUOTA_NL_BSOFTLONGWARN:
-			msg = "block quota exceeded too long";
+			msg = _("block quota exceeded too long");
 			break;
 		case QUOTA_NL_BSOFTWARN:
-			msg = "block quota exceeded";
+			msg = _("block quota exceeded");
+			break;
+		case QUOTA_NL_IHARDBELOW:
+			msg = _("got below file limit");
+			break;
+		case QUOTA_NL_ISOFTBELOW:
+			msg = _("got below file quota");
+			break;
+		case QUOTA_NL_BHARDBELOW:
+			msg = _("got below block limit");
+			break;
+		case QUOTA_NL_BSOFTBELOW:
+			msg = _("got below block quota");
 			break;
 		default:
-			msg = "unknown quota warning";
+			msg = _("unknown quota warning");
 	}
 	sprintf(warnbuf, "%s: %s %s %s.\r\n", level, type2name(warn->qtype), user, msg);
 	if (write_all(fd, warnbuf, strlen(warnbuf)) < 0)
-		errstr(_("Failed to write quota message for user %Lu to %s: %s\n"), (unsigned long long)warn->caused_id, dev, strerror(errno));
+		errstr(_("Failed to write quota message for user %llu to %s: %s\n"), (unsigned long long)warn->caused_id, dev, strerror(errno));
 	close(fd);
 }
 
