@@ -8,7 +8,7 @@
  *	New quota format implementation - Jan Kara <jack@suse.cz> - Sponsored by SuSE CR
  */
 
-#ident "$Id: quotacheck.c,v 1.57 2008/12/17 12:40:07 jkar8572 Exp $"
+#include "config.h"
 
 #include <dirent.h>
 #include <stdio.h>
@@ -62,24 +62,24 @@ struct dirs {
 #define BITS_SIZE 4		/* sizeof(bits) == 5 */
 #define BLIT_RATIO 10		/* Blit in just 1/10 of blit() calls */
 
-dev_t cur_dev;			/* Device we are working on */
-int files_done, dirs_done;
+static dev_t cur_dev;			/* Device we are working on */
+static int files_done, dirs_done;
 int flags, fmt = -1, cfmt;	/* Options from command line; Quota format to use spec. by user; Actual format to check */
-int uwant, gwant, ucheck, gcheck;	/* Does user want to check user/group quota; Do we check user/group quota? */
-char *mntpoint;			/* Mountpoint to check */
+static int uwant, gwant, ucheck, gcheck;	/* Does user want to check user/group quota; Do we check user/group quota? */
+static char *mntpoint;			/* Mountpoint to check */
 char *progname;
 struct util_dqinfo old_info[MAXQUOTAS];	/* Loaded infos */
 
-char extensions[MAXQUOTAS + 2][20] = INITQFNAMES;	/* Extensions depending on quota type */
-char *basenames[] = INITQFBASENAMES;	/* Names of quota files */
+static char extensions[MAXQUOTAS + 2][20] = INITQFNAMES;	/* Extensions depending on quota type */
+static char *basenames[] = INITQFBASENAMES;	/* Names of quota files */
 
 #ifdef DEBUG_MALLOC
-size_t malloc_mem = 0;
-size_t free_mem = 0;
+static size_t malloc_mem = 0;
+static size_t free_mem = 0;
 #endif
 
-struct dquot *dquot_hash[MAXQUOTAS][DQUOTHASHSIZE];
-struct dlinks *links_hash[MAXQUOTAS][DQUOTHASHSIZE];
+static struct dquot *dquot_hash[MAXQUOTAS][DQUOTHASHSIZE];
+static struct dlinks *links_hash[MAXQUOTAS][DQUOTHASHSIZE];
 
 /*
  * Ok check each memory allocation.
@@ -404,10 +404,6 @@ static void parse_options(int argcnt, char **argstr)
 		fputs(_("Bad number of arguments.\n"), stderr);
 		usage();
 	}
-	if (fmt == QF_XFS) {
-		fputs(_("XFS quota format needs no checking.\n"), stderr);
-		exit(0);
-	}
 	if (flags & FL_VERBOSE && flags & FL_DEBUG)
 		flags &= ~FL_VERBOSE;
 	if (!(flags & FL_ALL))
@@ -455,8 +451,10 @@ static int ext2_direct_scan(char *device)
 		return -1;
 	}
 
-	while ((long)i_num) {
-		if (inode.i_links_count) {
+	while (i_num) {
+		if ((i_num == EXT2_ROOT_INO ||
+		     i_num >= EXT2_FIRST_INO(fs->super)) &&
+		    inode.i_links_count) {
 			debug(FL_DEBUG, _("Found i_num %ld, blocks %ld\n"), (long)i_num, (long)inode.i_blocks);
 			if (flags & FL_VERBOSE)
 				blit(NULL);
@@ -623,7 +621,7 @@ static int process_file(struct mntent *mnt, int type)
 	debug(FL_DEBUG, _("Going to check %s quota file of %s\n"), type2name(type),
 	      mnt->mnt_dir);
 
-	if (kern_quota_on(mnt->mnt_fsname, type, (1 << cfmt)) > 0) {	/* Is quota enabled? */
+	if (kern_quota_on(mnt->mnt_fsname, type, cfmt) >= 0) {	/* Is quota enabled? */
 		if (!(flags & FL_FORCE)) {
 			if (flags & FL_INTERACTIVE) {
 				printf(_("Quota for %ss is enabled on mountpoint %s so quotacheck might damage the file.\n"), type2name(type), mnt->mnt_dir);
@@ -644,7 +642,7 @@ Please turn quotas off or use -f to force checking.\n"),
 	}
 
 	if (!(flags & FL_NEWFILE)) {	/* Need to buffer file? */
-		if (get_qf_name(mnt, type, (1 << cfmt), 0, &qfname) < 0) {
+		if (get_qf_name(mnt, type, cfmt, 0, &qfname) < 0) {
 			errstr(_("Cannot get quotafile name for %s\n"), mnt->mnt_fsname);
 			return -1;
 		}
@@ -664,18 +662,11 @@ Please turn quotas off or use -f to force checking.\n"),
 
 	ret = 0;
 	memset(old_info + type, 0, sizeof(old_info[type]));
-	switch (cfmt) {
-		case QF_TOONEW:
-			errstr(_("Too new quotafile format on %s\n"), mnt->mnt_fsname);
-			ret = -1;
-			break;
-		case QF_VFSOLD:
-			ret = v1_buffer_file(qfname, fd, type);
-			break;
-		case QF_VFSV0:
-			ret = v2_buffer_file(qfname, fd, type);
-			break;
-	}
+	if (is_tree_qfmt(cfmt))
+		ret = v2_buffer_file(qfname, fd, type, cfmt);
+	else
+		ret = v1_buffer_file(qfname, fd, type);
+
 	if (!(flags & FL_NEWFILE)) {
 		free(qfname);
 		close(fd);
@@ -694,8 +685,11 @@ static int rename_files(struct mntent *mnt, int type)
 	int fd;
 #endif
 
+	if (cfmt == QF_XFS) /* No renaming for xfs/gfs2 */
+		return 0;
+
 	debug(FL_DEBUG, _("Renaming new files to proper names.\n"));
-	if (get_qf_name(mnt, type, (1 << cfmt), 0, &filename) < 0)
+	if (get_qf_name(mnt, type, cfmt, 0, &filename) < 0)
 		die(2, _("Cannot get name of old quotafile on %s.\n"), mnt->mnt_dir);
 	if (stat(filename, &st) < 0) {	/* File doesn't exist? */
 		if (errno == ENOENT) {
@@ -721,9 +715,9 @@ static int rename_files(struct mntent *mnt, int type)
 	}
 	if (ioctl(fd, EXT2_IOC_GETFLAGS, &ext2_flags) < 0)
 		debug(FL_DEBUG, _("EXT2_IOC_GETFLAGS failed: %s\n"), strerror(errno));
-	/* IMMUTABLE flag set probably because system crashed and quota was not properly
-	 * turned off */
-	if (ext2_flags & EXT2_IMMUTABLE_FL) {
+	else if (ext2_flags & EXT2_IMMUTABLE_FL) {
+		/* IMMUTABLE flag set probably because system crashed and quota
+		 * was not properly turned off */
 		debug(FL_DEBUG | FL_VERBOSE, _("Quota file %s has IMMUTABLE flag set. Clearing.\n"), filename);
 		ext2_flags &= ~EXT2_IMMUTABLE_FL;
 		if (ioctl(fd, EXT2_IOC_SETFLAGS, &ext2_flags) < 0) {
@@ -792,35 +786,52 @@ static int dump_to_file(struct mntent *mnt, int type)
 	struct dquot *dquot;
 	uint i;
 	struct quota_handle *h;
+	unsigned int commit = 0;
 
 	debug(FL_DEBUG, _("Dumping gathered data for %ss.\n"), type2name(type));
-	if (!(h = new_io(mnt, type, cfmt))) {
-		errstr(_("Cannot initialize IO on new quotafile: %s\n"),
-			strerror(errno));
-		return -1;
-	}
-	if (!(flags & FL_NEWFILE)) {
-		h->qh_info.dqi_bgrace = old_info[type].dqi_bgrace;
-		h->qh_info.dqi_igrace = old_info[type].dqi_igrace;
-		if (cfmt == QF_VFSV0)
-			v2_merge_info(&h->qh_info, old_info + type);
-		mark_quotafile_info_dirty(h);
+	if (cfmt == QF_XFS) {
+		if (!(h = init_io(mnt, type, cfmt, IOI_READONLY))) {
+			errstr(_("Cannot initialize IO on xfs/gfs2 quotafile: %s\n"),
+			       strerror(errno));
+			return -1;
+		}
+	} else {
+		if (!(h = new_io(mnt, type, cfmt))) {
+			errstr(_("Cannot initialize IO on new quotafile: %s\n"),
+			       strerror(errno));
+			return -1;
+		}
+		if (!(flags & FL_NEWFILE)) {
+			h->qh_info.dqi_bgrace = old_info[type].dqi_bgrace;
+			h->qh_info.dqi_igrace = old_info[type].dqi_igrace;
+			if (is_tree_qfmt(cfmt))
+				v2_merge_info(&h->qh_info, old_info + type);
+			mark_quotafile_info_dirty(h);
+		}
 	}
 	for (i = 0; i < DQUOTHASHSIZE; i++)
 		for (dquot = dquot_hash[type][i]; dquot; dquot = dquot->dq_next) {
 			dquot->dq_h = h;
+			/* For XFS/GFS2, we don't bother with actually checking
+			 * what the usage value is in the internal quota file.
+			 * We simply attempt to update the usage for every quota
+			 * we find in the fs scan. The filesystem decides in the
+			 * quotactl handler whether to update the usage in the 
+			 * quota file or not.
+			 */
+			commit = cfmt == QF_XFS ? COMMIT_USAGE : COMMIT_ALL;
 			update_grace_times(dquot);
-			h->qh_ops->commit_dquot(dquot, COMMIT_ALL);
+			h->qh_ops->commit_dquot(dquot, commit);
 		}
 	if (end_io(h) < 0) {
 		errstr(_("Cannot finish IO on new quotafile: %s\n"), strerror(errno));
 		return -1;
 	}
 	debug(FL_DEBUG, _("Data dumped.\n"));
-	if (cfmt == kern_quota_on(mnt->mnt_fsname, type, 1 << cfmt)) {	/* Quota turned on? */
+	if (kern_quota_on(mnt->mnt_fsname, type, cfmt) >= 0) {	/* Quota turned on? */
 		char *filename;
 
-		if (get_qf_name(mnt, type, 1 << cfmt, NF_FORMAT, &filename) < 0)
+		if (get_qf_name(mnt, type, cfmt, NF_FORMAT, &filename) < 0)
 			errstr(_("Cannot find checked quota file for %ss on %s!\n"), type2name(type), mnt->mnt_fsname);
 		else {
 			if (quotactl(QCMD((kernel_iface == IFACE_GENERIC) ? Q_QUOTAOFF : Q_6_5_QUOTAOFF, type),
@@ -830,7 +841,7 @@ static int dump_to_file(struct mntent *mnt, int type)
 			else {
 				int ret;
 
-				/* Rename files - if it fails we cannot do anything better then just turn on quotas again */
+				/* Rename files - if it fails we cannot do anything better than just turn on quotas again */
 				rename_files(mnt, type);
 
 				if (kernel_iface == IFACE_GENERIC)
@@ -850,8 +861,9 @@ static int dump_to_file(struct mntent *mnt, int type)
 	return 0;
 }
 
-/* Substract space used by old quota file from usage */
-static void sub_quota_file(struct mntent *mnt, int qtype, int ftype)
+/* Substract space used by old quota file from usage.
+ * Return non-zero in case of failure, zero otherwise. */
+static int sub_quota_file(struct mntent *mnt, int qtype, int ftype)
 {
 	char *filename;
 	struct stat st;
@@ -860,15 +872,15 @@ static void sub_quota_file(struct mntent *mnt, int qtype, int ftype)
 	qid_t id;
 
 	debug(FL_DEBUG, _("Substracting space used by old %s quota file.\n"), type2name(ftype));
-	if (get_qf_name(mnt, ftype, 1 << cfmt, 0, &filename) < 0) {
-		debug(FL_VERBOSE, _("Old %s file not found. Usage will not be substracted.\n"), type2name(ftype));
-		return;
+	if (get_qf_name(mnt, ftype, cfmt, 0, &filename) < 0) {
+		debug(FL_VERBOSE, _("Old %s file name could not been determined. Usage will not be substracted.\n"), type2name(ftype));
+		return 0;
 	}
 
 	if (stat(filename, &st) < 0) {
-		debug(FL_VERBOSE, _("Cannot stat old %s quota file: %s\n"), type2name(ftype), strerror(errno));
+		debug(FL_VERBOSE, _("Cannot stat old %s quota file %s: %s. Usage will not be substracted.\n"), type2name(ftype), filename, strerror(errno));
 		free(filename);
-		return;
+		return 0;
 	}
 	qspace = getqsize(filename, &st);
 	free(filename);
@@ -879,18 +891,21 @@ static void sub_quota_file(struct mntent *mnt, int qtype, int ftype)
 		id = st.st_gid;
 	if ((d = lookup_dquot(id, qtype)) == NODQUOT) {
 		errstr(_("Quota structure for %s owning quota file not present! Something is really wrong...\n"), type2name(qtype));
-		return;
+		return -1;
 	}
 	d->dq_dqb.dqb_curinodes--;
 	d->dq_dqb.dqb_curspace -= qspace;
 	debug(FL_DEBUG, _("Substracted %lu bytes.\n"), (unsigned long)qspace);
+    return 0;
 }
 
-/* Buffer quotafile, run filesystem scan, dump quotafiles */
-static void check_dir(struct mntent *mnt)
+/* Buffer quotafile, run filesystem scan, dump quotafiles.
+ * Return non-zero value in case of failure, zero otherwise. */
+static int check_dir(struct mntent *mnt)
 {
 	struct stat st;
 	int remounted = 0;
+	int failed = 0;
 
 	if (lstat(mnt->mnt_dir, &st) < 0)
 		die(2, _("Cannot stat mountpoint %s: %s\n"), mnt->mnt_dir, strerror(errno));
@@ -898,6 +913,14 @@ static void check_dir(struct mntent *mnt)
 		die(2, _("Mountpoint %s is not a directory?!\n"), mnt->mnt_dir);
 	cur_dev = st.st_dev;
 	files_done = dirs_done = 0;
+	/*
+	 * For gfs2, we scan the fs first and then tell the kernel about the new usage.
+	 * So, there's no need to load any information. We also don't remount the
+	 * filesystem read-only because for a clustering filesystem it won't stop
+	 * modifications from other nodes anyway.
+	 */
+	if (cfmt == QF_XFS)
+		goto start_scan;
 	if (ucheck)
 		if (process_file(mnt, USRQUOTA) < 0)
 			ucheck = 0;
@@ -905,7 +928,7 @@ static void check_dir(struct mntent *mnt)
 		if (process_file(mnt, GRPQUOTA) < 0)
 			gcheck = 0;
 	if (!ucheck && !gcheck)	/* Nothing to check? */
-		return;
+		return 0;
 	if (!(flags & FL_NOREMOUNT)) {
 		/* Now we try to remount fs read-only to prevent races when scanning filesystem */
 		if (mount
@@ -915,12 +938,14 @@ static void check_dir(struct mntent *mnt)
 				printf(_("Cannot remount filesystem mounted on %s read-only. Counted values might not be right.\n"), mnt->mnt_dir);
 				if (!ask_yn(_("Should I continue"), 0)) {
 					printf(_("As you wish... Canceling check of this file.\n"));
+					failed = -1;
 					goto out;
 				}
 			}
 			else {
 				errstr(_("Cannot remount filesystem mounted on %s read-only so counted values might not be right.\n\
 Please stop all programs writing to filesystem or use -m flag to force checking.\n"), mnt->mnt_dir);
+				failed = -1;
 				goto out;
 			}
 		}
@@ -928,10 +953,11 @@ Please stop all programs writing to filesystem or use -m flag to force checking.
 			remounted = 1;
 		debug(FL_DEBUG, _("Filesystem remounted read-only\n"));
 	}
+start_scan:
 	debug(FL_VERBOSE, _("Scanning %s [%s] "), mnt->mnt_fsname, mnt->mnt_dir);
 #if defined(EXT2_DIRECT)
-	if (!strcmp(mnt->mnt_type, MNTTYPE_EXT2) || !strcmp(mnt->mnt_type, MNTTYPE_EXT3)) {
-		if (ext2_direct_scan(mnt->mnt_fsname) < 0)
+	if (!strcmp(mnt->mnt_type, MNTTYPE_EXT2) || !strcmp(mnt->mnt_type, MNTTYPE_EXT3) || !strcmp(mnt->mnt_type, MNTTYPE_NEXT3)) {
+		if ((failed = ext2_direct_scan(mnt->mnt_fsname)) < 0)
 			goto out;
 	}
 	else {
@@ -940,19 +966,19 @@ Please stop all programs writing to filesystem or use -m flag to force checking.
 #endif
 		if (flags & FL_VERYVERBOSE)
 			putchar('\n');
-		if (scan_dir(mnt->mnt_dir) < 0)
+		if ((failed = scan_dir(mnt->mnt_dir)) < 0)
 			goto out;
 	}
 	dirs_done++;
 	if (flags & FL_VERBOSE || flags & FL_VERYVERBOSE)
 		fputs(_("done\n"), stdout);
 	if (ucheck) {
-		sub_quota_file(mnt, USRQUOTA, USRQUOTA);
-		sub_quota_file(mnt, USRQUOTA, GRPQUOTA);
+		failed |= sub_quota_file(mnt, USRQUOTA, USRQUOTA);
+		failed |= sub_quota_file(mnt, USRQUOTA, GRPQUOTA);
 	}
 	if (gcheck) {
-		sub_quota_file(mnt, GRPQUOTA, USRQUOTA);
-		sub_quota_file(mnt, GRPQUOTA, GRPQUOTA);
+		failed |= sub_quota_file(mnt, GRPQUOTA, USRQUOTA);
+		failed |= sub_quota_file(mnt, GRPQUOTA, GRPQUOTA);
 	}
 	debug(FL_DEBUG | FL_VERBOSE, _("Checked %d directories and %d files\n"), dirs_done,
 	      files_done);
@@ -962,11 +988,12 @@ Please stop all programs writing to filesystem or use -m flag to force checking.
 		debug(FL_DEBUG, _("Filesystem remounted RW.\n"));
 	}
 	if (ucheck)
-		dump_to_file(mnt, USRQUOTA);
+		failed |= dump_to_file(mnt, USRQUOTA);
 	if (gcheck)
-		dump_to_file(mnt, GRPQUOTA);
+		failed |= dump_to_file(mnt, GRPQUOTA);
 out:
 	remove_list();
+	return failed;
 }
 
 /* Detect quota format from filename of present files */
@@ -976,6 +1003,11 @@ static int detect_filename_format(struct mntent *mnt, int type)
 	struct stat statbuf;
 	char namebuf[PATH_MAX];
 	int journal = 0;
+	int fmt;
+
+	if (strcmp(mnt->mnt_type, MNTTYPE_XFS) == 0 ||
+	    strcmp(mnt->mnt_type, MNTTYPE_GFS2) == 0)
+		return QF_XFS;
 
 	if (type == USRQUOTA) {
 		if ((option = hasmntopt(mnt, MNTOPT_USRQUOTA)))
@@ -998,7 +1030,6 @@ static int detect_filename_format(struct mntent *mnt, int type)
 	if (!option)
 		die(2, _("Cannot find quota option on filesystem %s with quotas!\n"), mnt->mnt_dir);
 	if (journal) {
-		int fmt;
 		char fmtbuf[64], *space;
 		
 		if (!(option = hasmntopt(mnt, MNTOPT_JQFMT))) {
@@ -1015,41 +1046,66 @@ jquota_err:
 		if (space-option > sizeof(fmtbuf))
 			goto jquota_err;
 		sstrncpy(fmtbuf, option+1, space-option);
-		if ((fmt = name2fmt(fmtbuf)) == QF_ERROR)
+		fmt = name2fmt(fmtbuf);
+		if (fmt == QF_ERROR)
 			goto jquota_err;
 		return fmt;
 	}
 	else if (*option == '=')	/* If the file name is specified we can't detect quota format from it... */
 		return -1;
 	snprintf(namebuf, PATH_MAX, "%s/%s.%s", mnt->mnt_dir, basenames[QF_VFSV0], extensions[type]);
-	if (!stat(namebuf, &statbuf))
-		return QF_VFSV0;
+	if (!stat(namebuf, &statbuf)) {
+		int fd = open(namebuf, O_RDONLY);
+		if (fd < 0)
+			return -1;
+		fmt = v2_detect_version(namebuf, fd, type);
+		close(fd);
+		return fmt;
+		
+	}
 	if (errno != ENOENT)
 		return -1;
 	snprintf(namebuf, PATH_MAX, "%s/%s.%s", mnt->mnt_dir, basenames[QF_VFSOLD], extensions[type]);
 	if (!stat(namebuf, &statbuf))
 		return QF_VFSOLD;
-	/* Old quota files don't exist, just create newest quotafile available */
-	if (kernel_formats & (1 << QF_VFSV0))
+	/* Old quota files don't exist, just create VFSv0 format if available */
+	if (kern_qfmt_supp(QF_VFSV0))
 		return QF_VFSV0;
-	if (kernel_formats & (1 << QF_VFSOLD))
+	if (kern_qfmt_supp(QF_VFSOLD))
 		return QF_VFSOLD;
 	return -1;
 }
 
-static void check_all(void)
+static int compatible_fs_qfmt(char *fstype, int fmt)
+{
+	/* We never check XFS, NFS, and filesystems supporting VFS metaformat */
+	if (!strcmp(fstype, MNTTYPE_XFS) || nfs_fstype(fstype) ||
+	    meta_qf_fstype(fstype))
+		return 0;
+	/* In all other cases we can pick a format... */
+	if (fmt == -1)
+		return 1;
+	/* XFS format is supported only by GFS2 */
+	if (fmt == QF_XFS)
+		return !strcmp(fstype, MNTTYPE_GFS2);
+	/* Anything but GFS2 supports all other formats */
+	return !!strcmp(fstype, MNTTYPE_GFS2);
+}
+
+/* Return 0 in case of success, non-zero otherwise. */
+static int check_all(void)
 {
 	struct mntent *mnt;
 	int checked = 0;
 	static int warned;
+	int failed = 0;
 
 	if (init_mounts_scan((flags & FL_ALL) ? 0 : 1, &mntpoint, 0) < 0)
 		die(2, _("Cannot initialize mountpoint scan.\n"));
 	while ((mnt = get_next_mount())) {
 		if (flags & FL_ALL && flags & FL_NOROOT && !strcmp(mnt->mnt_dir, "/"))
 			continue;
-		if (!strcmp(mnt->mnt_type, MNTTYPE_XFS) || nfs_fstype(mnt->mnt_type) ||
-		    meta_qf_fstype(mnt->mnt_type)) {
+		if (!compatible_fs_qfmt(mnt->mnt_type, fmt)) {
 			debug(FL_DEBUG | FL_VERBOSE, _("Skipping %s [%s]\n"), mnt->mnt_fsname, mnt->mnt_dir);
 			continue;
 		}
@@ -1065,9 +1121,11 @@ static void check_all(void)
 		if (!ucheck && !gcheck)
 			continue;
 		if (cfmt == -1) {
-			if ((cfmt = detect_filename_format(mnt, ucheck ? USRQUOTA : GRPQUOTA)) == -1) {
+			cfmt = detect_filename_format(mnt, ucheck ? USRQUOTA : GRPQUOTA);
+			if (cfmt == -1) {
 				errstr(_("Cannot guess format from filename on %s. Please specify format on commandline.\n"),
 					mnt->mnt_fsname);
+				failed = -1;
 				continue;
 			}
 			debug(FL_DEBUG, _("Detected quota format %s\n"), fmt2name(cfmt));
@@ -1077,6 +1135,7 @@ static void check_all(void)
 		    !hasmntopt(mnt, MNTOPT_GRPJQUOTA) && !warned &&
 		    (!strcmp(mnt->mnt_type, MNTTYPE_EXT3) ||
 		     !strcmp(mnt->mnt_type, MNTTYPE_EXT4) ||
+		     !strcmp(mnt->mnt_type, MNTTYPE_NEXT3) ||
 		     !strcmp(mnt->mnt_type, MNTTYPE_EXT4DEV) ||
 		     !strcmp(mnt->mnt_type, MNTTYPE_REISER))) {
 			struct utsname stats;
@@ -1104,25 +1163,30 @@ static void check_all(void)
 		}
 
 		checked++;
-		check_dir(mnt);
+		failed |= check_dir(mnt);
 	}
 	end_mounts_scan();
-	if (!checked && (!(flags & FL_ALL) || flags & (FL_VERBOSE | FL_DEBUG)))
+	if (!checked && (!(flags & FL_ALL) || flags & (FL_VERBOSE | FL_DEBUG))) {
 		errstr(_("Cannot find filesystem to check or filesystem not mounted with quota option.\n"));
+		failed = -1;
+    }
+    return failed;
 }
 
 int main(int argc, char **argv)
 {
+	int failed;
+
 	gettexton();
 	progname = basename(argv[0]);
 
 	parse_options(argc, argv);
 	init_kernel_interface();
 
-	check_all();
+	failed = check_all();
 #ifdef DEBUG_MALLOC
 	errstr(_("Allocated %d bytes memory\nFree'd %d bytes\nLost %d bytes\n"),
 		malloc_mem, free_mem, malloc_mem - free_mem);
 #endif
-	return 0;
+	return (failed ? EXIT_FAILURE : EXIT_SUCCESS);
 }
