@@ -5,6 +5,8 @@
  *	Jan Kara <jack@suse.cz> - sponsored by SuSE CR
  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
@@ -41,6 +43,7 @@ struct quota_handle *init_io(struct mntent *mnt, int type, int fmt, int flags)
 	int fd = -1, kernfmt;
 	struct quota_handle *h = smalloc(sizeof(struct quota_handle));
 	const char *mnt_fsname = NULL;
+	int nameflag;
 
 	if (!hasquota(mnt, type, 0))
 		goto out_handle;
@@ -57,6 +60,7 @@ struct quota_handle *init_io(struct mntent *mnt, int type, int fmt, int flags)
 	sstrncpy(h->qh_quotadev, mnt_fsname, sizeof(h->qh_quotadev));
 	free((char *)mnt_fsname);
 	sstrncpy(h->qh_fstype, mnt->mnt_type, MAX_FSTYPE_LEN);
+	sstrncpy(h->qh_dir, mnt->mnt_dir, PATH_MAX);
 	if (nfs_fstype(mnt->mnt_type)) {	/* NFS filesystem? */
 		if (fmt != -1 && fmt != QF_RPC) {	/* User wanted some other format? */
 			errstr(_("Only RPC quota format is allowed on NFS filesystem.\n"));
@@ -71,9 +75,13 @@ struct quota_handle *init_io(struct mntent *mnt, int type, int fmt, int flags)
 		errstr(_("RPC quota format not compiled.\n"));
 		goto out_handle;
 #endif
+	} else if (fmt == QF_RPC) {
+		errstr(_("RPC quota format specified for non-NFS filesystem.\n"));
+		goto out_handle;
 	}
 
-	if (!strcmp(mnt->mnt_type, MNTTYPE_XFS)) {	/* XFS filesystem? */
+	if (!strcmp(mnt->mnt_type, MNTTYPE_XFS) ||	/* XFS filesystem? */
+	    !strcmp(mnt->mnt_type, MNTTYPE_GFS2)) {	/* XFS filesystem? */
 		if (fmt != -1 && fmt != QF_XFS) {	/* User wanted some other format? */
 			errstr(_("Only XFS quota format is allowed on XFS filesystem.\n"));
 			goto out_handle;
@@ -89,20 +97,56 @@ struct quota_handle *init_io(struct mntent *mnt, int type, int fmt, int flags)
 		errstr(_("XFS quota allowed only on XFS filesystem.\n"));
 		goto out_handle;
 	}
-	if (kernel_formats > 0 && (fmt == -1 || (1 << fmt) & kernel_formats)) {	/* Quota compiled and desired format available? */
+	if (kern_qfmt_supp(fmt)) {	/* Quota compiled and desired format available? */
 		/* Quota turned on? */
-		kernfmt = kern_quota_on(h->qh_quotadev, type, fmt == -1 ? kernel_formats : (1 << fmt));
+		kernfmt = kern_quota_on(h->qh_quotadev, type, fmt);
 		if (kernfmt >= 0) {
 			h->qh_io_flags |= IOFL_QUOTAON;
 			fmt = kernfmt;	/* Default is kernel used format */
 		}
 	}
-	if ((fmt = get_qf_name(mnt, type, (fmt == -1) ? ((1 << QF_VFSOLD) | (1 << QF_VFSV0)) : (1 << fmt),
-	    (!QIO_ENABLED(h) || flags & IOI_OPENFILE) ? NF_FORMAT : 0, &qfname)) < 0) {
-		errstr(_("Quota file not found or has wrong format.\n"));
-		goto out_handle;
+
+	if (meta_qf_fstype(mnt->mnt_type)) {
+		if (!QIO_ENABLED(h)) {
+			errstr(_("Quota not supported by the filesystem.\n"));
+			goto out_handle;
+		}
+		h->qh_fd = -1;
+		h->qh_fmt = fmt;
+		goto set_ops;
 	}
-	if (!QIO_ENABLED(h) || flags & IOI_OPENFILE) {	/* Need to open file? */
+
+	nameflag = (!QIO_ENABLED(h) || flags & IOI_INITSCAN) ? NF_FORMAT : 0;
+	if (fmt == -1) {
+		/* Let's try any VFSv0 quota format... */
+		if (get_qf_name(mnt, type, QF_VFSV0, nameflag, &qfname) >= 0)
+			fmt = QF_VFSV0;
+		/* And then VFSv1 quota format... */
+		else if (get_qf_name(mnt, type, QF_VFSV1, nameflag, &qfname) >= 0)
+			fmt = QF_VFSV1;
+		/* And then old quota format... */
+		else if (get_qf_name(mnt, type, QF_VFSOLD, nameflag, &qfname) >= 0)
+			fmt = QF_VFSOLD;
+		else {	/* Don't know... */
+			errstr(_("Cannot find any quota file to work on.\n"));
+			goto out_handle;
+		}
+	} else {
+		if (get_qf_name(mnt, type, fmt, nameflag, &qfname) < 0) {
+			errstr(_("Quota file not found or has wrong format.\n"));
+			goto out_handle;
+		}
+	}
+	if (!QIO_ENABLED(h) || flags & IOI_INITSCAN) {	/* Need to open file? */
+		if (QIO_ENABLED(h)) {	/* Kernel uses same file? */
+			unsigned int cmd =
+				(kernel_iface == IFACE_GENERIC) ? Q_SYNC : Q_6_5_SYNC;
+			if (quotactl(QCMD(cmd, h->qh_type), h->qh_quotadev,
+				     0, NULL) < 0) {
+				die(4, _("Cannot sync quotas on device %s: %s\n"),
+				    h->qh_quotadev, strerror(errno));
+			}
+		}
 		/* We still need to open file for operations like 'repquota' */
 		if ((fd = open(qfname, QIO_RO(h) ? O_RDONLY : O_RDWR)) < 0) {
 			errstr(_("Cannot open quotafile %s: %s\n"),
@@ -113,18 +157,20 @@ struct quota_handle *init_io(struct mntent *mnt, int type, int fmt, int flags)
 		/* Init handle */
 		h->qh_fd = fd;
 		h->qh_fmt = fmt;
-	}
-	else {
+	} else {
 		h->qh_fd = -1;
 		h->qh_fmt = fmt;
 	}
 	free(qfname);	/* We don't need it anymore */
 	qfname = NULL;
 
-	if (h->qh_fmt == QF_VFSOLD)
+set_ops:
+	if (fmt == QF_VFSOLD)
 		h->qh_ops = &quotafile_ops_1;
-	else if (h->qh_fmt == QF_VFSV0)
+	else if (is_tree_qfmt(fmt))
 		h->qh_ops = &quotafile_ops_2;
+	else if (fmt == QF_META)
+		h->qh_ops = &quotafile_ops_meta;
 	memset(&h->qh_info, 0, sizeof(h->qh_info));
 
 	if (h->qh_ops->init_io && h->qh_ops->init_io(h) < 0) {
@@ -154,13 +200,13 @@ struct quota_handle *new_io(struct mntent *mnt, int type, int fmt)
 	char namebuf[PATH_MAX];
 
 	if (fmt == -1)
-		fmt = QF_VFSV0;	/* Use the newest format */
-	else if (fmt == QF_RPC || fmt == QF_XFS) {
+		fmt = QF_VFSV0;
+	else if (fmt == QF_RPC || fmt == QF_XFS || meta_qf_fstype(mnt->mnt_type)) {
 		errstr(_("Creation of %s quota format is not supported.\n"),
-			fmt == QF_RPC ? "RPC" : "XFS");
+			fmt2name(fmt));
 		return NULL;
 	}
-	if (get_qf_name(mnt, type, (1 << fmt), 0, &qfname) < 0)
+	if (get_qf_name(mnt, type, fmt, 0, &qfname) < 0)
 		return NULL;
 	sstrncpy(namebuf, qfname, PATH_MAX);
 	sstrncat(namebuf, ".new", PATH_MAX);
@@ -177,8 +223,11 @@ struct quota_handle *new_io(struct mntent *mnt, int type, int fmt)
 	h->qh_fd = fd;
 	h->qh_io_flags = 0;
 	sstrncpy(h->qh_quotadev, mnt_fsname, sizeof(h->qh_quotadev));
+	sstrncpy(h->qh_fstype, mnt->mnt_type, MAX_FSTYPE_LEN);
+	sstrncpy(h->qh_dir, mnt->mnt_dir, PATH_MAX);
 	free((char *)mnt_fsname);
 	h->qh_type = type;
+	h->qh_fmt = fmt;
 	memset(&h->qh_info, 0, sizeof(h->qh_info));
 	if (fmt == QF_VFSOLD)
 		h->qh_ops = &quotafile_ops_1;
@@ -192,7 +241,7 @@ struct quota_handle *new_io(struct mntent *mnt, int type, int fmt)
 		goto out_fd;
 	}
 	return h;
-      out_fd:
+out_fd:
 	close(fd);
 	return NULL;
 }
@@ -228,3 +277,27 @@ struct dquot *get_empty_dquot(void)
 	dquot->dq_id = -1;
 	return dquot;
 }
+
+/*
+ *	Check whether values in current dquot can be stored on disk
+ */
+int check_dquot_range(struct dquot *dquot)
+{
+	struct util_dqinfo *info = &dquot->dq_h->qh_info;
+
+	if (dquot->dq_dqb.dqb_bhardlimit > info->dqi_max_b_limit ||
+	    dquot->dq_dqb.dqb_bsoftlimit > info->dqi_max_b_limit ||
+	    dquot->dq_dqb.dqb_ihardlimit > info->dqi_max_i_limit ||
+	    dquot->dq_dqb.dqb_isoftlimit > info->dqi_max_i_limit) {
+		errstr(_("Trying to set quota limits out of range "
+				 "supported by quota format on %s.\n"), dquot->dq_h->qh_quotadev);
+		return -1;
+	}
+	if (dquot->dq_dqb.dqb_curinodes > info->dqi_max_i_usage ||
+	    dquot->dq_dqb.dqb_curspace > info->dqi_max_b_usage) {
+		errstr(_("Trying to set quota usage out of range "
+			 "supported by quota format on %s.\n"), dquot->dq_h->qh_quotadev);
+		return -1;
+	}
+	return 0;
+}
