@@ -4,12 +4,13 @@
  *
  */
 
-#ifndef _QUOTAIO_H
-#define _QUOTAIO_H
+#ifndef GUARD_QUOTAIO_H
+#define GUARD_QUOTAIO_H
 
 #include <limits.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdint.h>
 
 #include "quota.h"
 #include "mntopt.h"
@@ -18,38 +19,32 @@
 #include "dqblk_rpc.h"
 #include "dqblk_xfs.h"
 
-/* Latest known versions */
-#define INITKNOWNVERSIONS {\
-	0,\
-	0\
-}
-
-#define QUOTAFORMATS 4
+#define QUOTAFORMATS 6
 
 #define INITQFBASENAMES {\
 	"quota",\
 	"aquota",\
+	"aquota",\
+	"",\
+	"",\
 	"",\
-	""\
-}
-
-#define INITQFMTNAMES {\
-	"vfsold",\
-	"vfsv0",\
-	"rpc",\
-	"xfs"\
 }
 
 #define MAX_FSTYPE_LEN 16		/* Maximum length of filesystem type name */
 
 /* Values for format handling */
-#define QF_UNKNOWN -3		/* Format cannot be detected from filename */
-#define QF_TOONEW -2		/* Quota format is too new to handle */
 #define QF_ERROR -1		/* There was error while detecting format (maybe unknown format...) */
 #define QF_VFSOLD 0		/* Old quota format */
-#define QF_VFSV0 1		/* New quota format - version 0 */
-#define QF_RPC 2		/* RPC should be used on given filesystem */
-#define QF_XFS 3		/* XFS quota format */
+#define QF_VFSV0 1		/* Quota files with tree quota format */
+#define QF_VFSV1 2		/* Quota files with 64-bit tree quota format */
+#define QF_RPC 3		/* RPC should be used on given filesystem */
+#define QF_XFS 4		/* XFS quota format */
+#define QF_META 5		/* Quota files are hidden, we don't care about the format */
+
+static inline int is_tree_qfmt(int fmt)
+{
+	return fmt == QF_VFSV0 || fmt == QF_VFSV1;
+}
 
 /*
  * Definitions for disk quotas imposed on the average user
@@ -75,6 +70,10 @@ struct quotafile_ops;
 struct util_dqinfo {
 	time_t dqi_bgrace;	/* Block grace time for given quotafile */
 	time_t dqi_igrace;	/* Inode grace time for given quotafile */
+	uint64_t dqi_max_b_limit;	/* Maximal block limit storable in current format */
+	uint64_t dqi_max_i_limit;	/* Maximal inode limit storable in current format */
+	uint64_t dqi_max_b_usage;	/* Maximal block usage storable in current format */
+	uint64_t dqi_max_i_usage;	/* Maximal inode usage storable in current format */
 	union {
 		struct v2_mem_dqinfo v2_mdqi;
 		struct xfs_mem_dqinfo xfs_mdqi;
@@ -86,6 +85,7 @@ struct quota_handle {
 	int qh_fd;		/* Handle of file (-1 when IOFL_QUOTAON) */
 	int qh_io_flags;	/* IO flags for file */
 	char qh_quotadev[PATH_MAX];	/* Device file is for */
+	char qh_dir[PATH_MAX];		/* Directory filesystem is mounted at */
 	char qh_fstype[MAX_FSTYPE_LEN];	/* Type of the filesystem on qh_quotadev */
 	int qh_type;		/* Type of quotafile */
 	int qh_fmt;		/* Quotafile format */
@@ -142,7 +142,7 @@ struct dquot {
 
 /* Structure of quotafile operations */
 struct quotafile_ops {
-	int (*check_file) (int fd, int type);		/* Check whether quotafile is in our format */
+	int (*check_file) (int fd, int type, int fmt);	/* Check whether quotafile is in our format */
 	int (*init_io) (struct quota_handle * h);	/* Open quotafile */
 	int (*new_io) (struct quota_handle * h);	/* Create new quotafile */
 	int (*end_io) (struct quota_handle * h);	/* Write all changes and close quotafile */
@@ -153,6 +153,9 @@ struct quotafile_ops {
 	int (*report) (struct quota_handle * h, int verbose);	/* Function called after 'repquota' to print format specific file information */
 };
 
+/* This might go into a special header file but that sounds a bit silly... */
+extern struct quotafile_ops quotafile_ops_meta;
+
 static inline void mark_quotafile_info_dirty(struct quota_handle *h)
 {
 	h->qh_io_flags |= IOFL_INFODIRTY;
@@ -173,4 +176,7 @@ int end_io(struct quota_handle *h);
 /* Get empty quota structure */
 struct dquot *get_empty_dquot(void);
 
-#endif /* _QUOTAIO_H */
+/* Check whether values in current dquot can be stored on disk */
+int check_dquot_range(struct dquot *dquot);
+
+#endif /* GUARD_QUOTAIO_H */
