@@ -32,9 +32,7 @@
  * SUCH DAMAGE.
  */
 
-#ident "$Copyright: (c) 1980, 1990 Regents of the University of California $"
-#ident "$Copyright: All rights reserved. $"
-#ident "$Id: quotaon.c,v 1.26 2008/12/17 12:40:07 jkar8572 Exp $"
+#include "config.h"
 
 /*
  * Turn quota on/off for a filesystem.
@@ -44,6 +42,7 @@
 #include <getopt.h>
 #include <string.h>
 #include <stdlib.h>
+#include <stdarg.h>
 
 #include "quotaon.h"
 #include "quota.h"
@@ -56,17 +55,17 @@
 #define FL_STAT 16
 #define FL_OFF 32
 
-int flags, fmt = -1;
+static int flags, fmt = -1;
 char *progname;
-char **mntpoints;
-int mntcnt;
-char *xarg = NULL;
+static char **mntpoints;
+static int mntcnt;
+static char *xarg = NULL;
 
 static void usage(void)
 {
 	errstr(_("Usage:\n\t%s [-guvp] [-F quotaformat] [-x state] -a\n\
 \t%s [-guvp] [-F quotaformat] [-x state] filesys ...\n\n\
--a, --all                turn quotas on for all filesystems\n\
+-a, --all                turn quotas %s for all filesystems\n\
 -f, --off                turn quotas off\n\
 -u, --user               operate on user quotas\n\
 -g, --group              operate on group quotas\n\
@@ -75,7 +74,8 @@ static void usage(void)
 -F, --format=formatname  operate on specific quota format\n\
 -v, --verbose            print more messages\n\
 -h, --help               display this help text and exit\n\
--V, --version            display version information and exit\n"), progname, progname);
+-V, --version            display version information and exit\n"),
+ progname, progname, strcmp(progname, "quotaon") ? _("off") : _("on"));
 	exit(1);
 }
 
@@ -147,118 +147,17 @@ static void parse_options(int argcnt, char **argstr)
 	}
 }
 
-/*
- *	For both VFS quota formats, need to pass in the quota file;
- *	for XFS quota manager, pass on the -x command line option.
- */
-static int newstate(struct mntent *mnt, int type, char *extra)
+int pinfo(char *fmt, ...)
 {
-	int sflags, ret = 0, usefmt;
-	newstate_t *statefunc;
+	va_list arg;
+	int ret;
 
-	sflags = flags & FL_OFF ? STATEFLAG_OFF : STATEFLAG_ON;
-	if (flags & FL_VERBOSE)
-		sflags |= STATEFLAG_VERBOSE;
-	if (flags & FL_ALL)
-		sflags |= STATEFLAG_ALL;
-
-	if (!strcmp(mnt->mnt_type, MNTTYPE_XFS)) {	/* XFS filesystem has special handling... */
-		if (!(kernel_formats & (1 << QF_XFS))) {
-			errstr(_("Cannot change state of XFS quota. It's not compiled in kernel.\n"));
-			return 1;
-		}
-		if (kernel_formats & (1 << QF_XFS) &&
-		    ((flags & FL_OFF && (kern_quota_on(mnt->mnt_fsname, USRQUOTA, 1 << QF_XFS)
-		    || kern_quota_on(mnt->mnt_fsname, GRPQUOTA, 1 << QF_XFS)))
-		    || (!(flags & FL_OFF) && kern_quota_on(mnt->mnt_fsname, type, 1 << QF_XFS))))
-			ret = xfs_newstate(mnt, type, extra, sflags);
-	}
-	else if (meta_qf_fstype(mnt->mnt_type)) {
-		if (!hasquota(mnt, type, 0))
-			return 0;
-		/* Must be non-empty because empty path is always invalid. */
-		ret = v2_newstate(mnt, type, ".", sflags);
-	}
-	else {
-		if (!hasquota(mnt, type, 0))
-			return 0;
-		usefmt = get_qf_name(mnt, type, fmt == -1 ? kernel_formats : (1 << fmt), NF_FORMAT, &extra);
-		if (usefmt < 0) {
-			errstr(_("Cannot find quota file on %s [%s] to turn quotas on/off.\n"), mnt->mnt_dir, mnt->mnt_fsname);
-			return 1;
-		}
-		statefunc = (usefmt == QF_VFSV0) ? v2_newstate : v1_newstate;
-		ret = statefunc(mnt, type, extra, sflags);
-		free(extra);
-	}
-	return ret;
-}
-
-/* Print state of quota (on/off) */
-static int print_state(struct mntent *mnt, int type)
-{
-	int on = 0;
-
-	if (!strcmp(mnt->mnt_type, MNTTYPE_XFS)) {
-		if (kernel_formats & (1 << QF_XFS))
-			on = kern_quota_on(mnt->mnt_fsname, type, 1 << QF_XFS) != -1;
-	}
-	else if (kernel_iface == IFACE_GENERIC)
-		/* PSz 28 Apr 04  Have V0 and OLD set, try both */
-		on = kern_quota_on(mnt->mnt_fsname, type, kernel_formats) != -1;
-	else if (kernel_formats & (1 << QF_VFSV0))
-		on = kern_quota_on(mnt->mnt_fsname, type, 1 << QF_VFSV0) != -1;
-	else if (kernel_formats & (1 << QF_VFSOLD))
-		on = kern_quota_on(mnt->mnt_fsname, type, 1 << QF_VFSOLD) != -1;
-
-	printf(_("%s quota on %s (%s) is %s\n"), type2name(type), mnt->mnt_dir, mnt->mnt_fsname,
-	  on ? _("on") : _("off"));
-	
-	return on;
-}
-
-/*
- *	Enable/disable VFS quota on given filesystem
- */
-static int quotaonoff(char *quotadev, char *quotadir, char *quotafile, int type, int fmt, int flags)
-{
-	int qcmd, kqf;
-
-	if (flags & STATEFLAG_OFF) {
-		if (kernel_iface == IFACE_GENERIC)
-			qcmd = QCMD(Q_QUOTAOFF, type);
-		else
-			qcmd = QCMD(Q_6_5_QUOTAOFF, type);
-		if (quotactl(qcmd, quotadev, 0, NULL) < 0) {
-			errstr(_("quotactl on %s [%s]: %s\n"), quotadev, quotadir, strerror(errno));
-			return 1;
-		}
-		if (flags & STATEFLAG_VERBOSE)
-			printf(_("%s [%s]: %s quotas turned off\n"), quotadev, quotadir, type2name(type));
+	if (!(flags & FL_VERBOSE))
 		return 0;
-	}
-	if (kernel_iface == IFACE_GENERIC) {
-		qcmd = QCMD(Q_QUOTAON, type);
- 		kqf = util2kernfmt(fmt);
-	}
-	else {
-		qcmd = QCMD(Q_6_5_QUOTAON, type);
-		kqf = 0;
-	}
-	if (quotactl(qcmd, quotadev, kqf, (void *)quotafile) < 0) {
-		if (errno == ENOENT)
-			errstr(_("cannot find %s on %s [%s]\n"), quotafile, quotadev, quotadir);
-		else
-			errstr(_("using %s on %s [%s]: %s\n"), quotafile, quotadev, quotadir, strerror(errno));
-		if (errno == EINVAL)
-			errstr(_("Maybe create new quota files with quotacheck(8)?\n"));
-		else if (errno == ESRCH)
-			errstr(_("Quota format not supported in kernel.\n"));
-		return 1;
-	}
-	if (flags & STATEFLAG_VERBOSE)
-		printf(_("%s [%s]: %s quotas turned on\n"), quotadev, quotadir, type2name(type));
-	return 0;
+	va_start(arg, fmt);
+	ret = vprintf(fmt, arg);
+	va_end(arg);
+	return ret;
 }
 
 /*
@@ -287,18 +186,60 @@ static int quotarsquashonoff(const char *quotadev, int type, int flags)
 		errstr(_("set root_squash on %s: %s\n"), quotadev, strerror(errno));
 		return 1;
 	}
-	if ((flags & STATEFLAG_VERBOSE) && (flags & STATEFLAG_OFF))
-		printf(_("%s: %s root_squash turned off\n"), quotadev, type2name(type));
-	else if ((flags & STATEFLAG_VERBOSE) && (flags & STATEFLAG_ON))
-		printf(_("%s: %s root_squash turned on\n"), quotadev, type2name(type));
+	if (flags & STATEFLAG_OFF)
+		pinfo(_("%s: %s root_squash turned off\n"), quotadev, type2name(type));
+	else if (flags & STATEFLAG_ON)
+		pinfo(_("%s: %s root_squash turned on\n"), quotadev, type2name(type));
 #endif
 	return 0;
 }
 
+/*
+ *	Enable/disable VFS quota on given filesystem
+ */
+static int quotaonoff(char *quotadev, char *quotadir, char *quotafile, int type, int fmt, int flags)
+{
+	int qcmd, kqf;
+
+	if (flags & STATEFLAG_OFF) {
+		if (kernel_iface == IFACE_GENERIC)
+			qcmd = QCMD(Q_QUOTAOFF, type);
+		else
+			qcmd = QCMD(Q_6_5_QUOTAOFF, type);
+		if (quotactl(qcmd, quotadev, 0, NULL) < 0) {
+			errstr(_("quotactl on %s [%s]: %s\n"), quotadev, quotadir, strerror(errno));
+			return 1;
+		}
+		pinfo(_("%s [%s]: %s quotas turned off\n"), quotadev, quotadir, type2name(type));
+		return 0;
+	}
+	if (kernel_iface == IFACE_GENERIC) {
+		qcmd = QCMD(Q_QUOTAON, type);
+ 		kqf = util2kernfmt(fmt);
+	}
+	else {
+		qcmd = QCMD(Q_6_5_QUOTAON, type);
+		kqf = 0;
+	}
+	if (quotactl(qcmd, quotadev, kqf, (void *)quotafile) < 0) {
+		if (errno == ENOENT)
+			errstr(_("cannot find %s on %s [%s]\n"), quotafile, quotadev, quotadir);
+		else
+			errstr(_("using %s on %s [%s]: %s\n"), quotafile, quotadev, quotadir, strerror(errno));
+		if (errno == EINVAL)
+			errstr(_("Maybe create new quota files with quotacheck(8)?\n"));
+		else if (errno == ESRCH)
+			errstr(_("Quota format not supported in kernel.\n"));
+		return 1;
+	}
+	pinfo(_("%s [%s]: %s quotas turned on\n"), quotadev, quotadir, type2name(type));
+	return 0;
+}
+
 /*
  *	Enable/disable quota/rootsquash on given filesystem (version 1)
  */
-int v1_newstate(struct mntent *mnt, int type, char *file, int flags)
+static int v1_newstate(struct mntent *mnt, int type, char *file, int flags, int fmt)
 {
 	int errs = 0;
 	const char *dev = get_device_name(mnt->mnt_fsname);
@@ -316,22 +257,107 @@ int v1_newstate(struct mntent *mnt, int type, char *file, int flags)
 }
 
 /*
- *	Enable/disable quota on given filesystem (version 2 quota)
+ *	Enable/disable quota on given filesystem (generic VFS quota)
  */
-int v2_newstate(struct mntent *mnt, int type, char *file, int flags)
+static int v2_newstate(struct mntent *mnt, int type, char *file, int flags, int fmt)
 {
 	const char *dev = get_device_name(mnt->mnt_fsname);
 	int errs = 0;
 
 	if (!dev)
 		return 1;
-
 	if (hasquota(mnt, type, 0))
-		errs = quotaonoff((char *)dev, mnt->mnt_dir, file, type, QF_VFSV0, flags);
+		errs = quotaonoff((char *)dev, mnt->mnt_dir, file, type, fmt, flags);
 	free((char *)dev);
 	return errs;
 }
 
+/*
+ *	For both VFS quota formats, need to pass in the quota file;
+ *	for XFS quota manager, pass on the -x command line option.
+ */
+static int newstate(struct mntent *mnt, int type, char *extra)
+{
+	int sflags, ret = 0;
+
+	sflags = flags & FL_OFF ? STATEFLAG_OFF : STATEFLAG_ON;
+	if (flags & FL_ALL)
+		sflags |= STATEFLAG_ALL;
+
+	if (!strcmp(mnt->mnt_type, MNTTYPE_GFS2)) {
+		errstr(_("Cannot change state of GFS2 quota.\n"));
+		return 1;
+	} else if (!strcmp(mnt->mnt_type, MNTTYPE_XFS)) {	/* XFS filesystem has special handling... */
+		if (!kern_qfmt_supp(QF_XFS)) {
+			errstr(_("Cannot change state of XFS quota. It's not compiled in kernel.\n"));
+			return 1;
+		}
+		ret = xfs_newstate(mnt, type, extra, sflags);
+	}
+	else if (meta_qf_fstype(mnt->mnt_type)) {
+		if (!hasquota(mnt, type, 0))
+			return 0;
+		/* Must be non-empty because empty path is always invalid. */
+		ret = v2_newstate(mnt, type, ".", sflags, QF_VFSV0);
+	}
+	else {
+		int usefmt;
+
+		if (!hasquota(mnt, type, 0))
+			return 0;
+		if (fmt == -1) {
+			if (get_qf_name(mnt, type, QF_VFSV0,
+					NF_FORMAT, &extra) >= 0)
+				usefmt = QF_VFSV0;
+			else if (get_qf_name(mnt, type, QF_VFSV1,
+					NF_FORMAT, &extra) >= 0)
+				usefmt = QF_VFSV1;
+			else if (get_qf_name(mnt, type, QF_VFSOLD,
+					NF_FORMAT, &extra) >= 0)
+				usefmt = QF_VFSOLD;
+			else {
+				errstr(_("Cannot find quota file on %s [%s] to turn quotas on/off.\n"), mnt->mnt_dir, mnt->mnt_fsname);
+				return 1;
+			}
+		} else {
+			if (get_qf_name(mnt, type, fmt, NF_FORMAT, &extra) < 0) {
+				errstr(_("Quota file on %s [%s] does not exist or has wrong format.\n"), mnt->mnt_dir, mnt->mnt_fsname);
+				return 1;
+			}
+			usefmt = fmt;
+		}
+		if (is_tree_qfmt(usefmt))
+			ret = v2_newstate(mnt, type, extra, sflags, usefmt);
+		else
+			ret = v1_newstate(mnt, type, extra, sflags, QF_VFSOLD);
+		free(extra);
+	}
+	return ret;
+}
+
+/* Print state of quota (on/off) */
+static int print_state(struct mntent *mnt, int type)
+{
+	int on = 0;
+
+	if (!strcmp(mnt->mnt_type, MNTTYPE_XFS) ||
+	    !strcmp(mnt->mnt_type, MNTTYPE_GFS2)) {
+		if (kern_qfmt_supp(QF_XFS))
+			on = kern_quota_on(mnt->mnt_fsname, type, QF_XFS) != -1;
+	}
+	else if (kernel_iface == IFACE_GENERIC)
+		on = kern_quota_on(mnt->mnt_fsname, type, -1) != -1;
+	else if (kern_qfmt_supp(QF_VFSV0))
+		on = kern_quota_on(mnt->mnt_fsname, type, QF_VFSV0) != -1;
+	else if (kern_qfmt_supp(QF_VFSOLD))
+		on = kern_quota_on(mnt->mnt_fsname, type, QF_VFSOLD) != -1;
+
+	printf(_("%s quota on %s (%s) is %s\n"), type2name(type), mnt->mnt_dir, mnt->mnt_fsname,
+	  on ? _("on") : _("off"));
+	
+	return on;
+}
+
 int main(int argc, char **argv)
 {
 	struct mntent *mnt;
@@ -348,9 +374,9 @@ int main(int argc, char **argv)
 	parse_options(argc, argv);
 
 	init_kernel_interface();
-	if (fmt != -1 && !(kernel_formats & (1 << fmt)))
+	if (fmt != -1 && !kern_qfmt_supp(fmt))
 		die(1, _("Required format %s not supported by kernel.\n"), fmt2name(fmt));
-	else if (!kernel_formats)
+	else if (!kern_qfmt_supp(-1))
 		errstr(_("Warning: No quota format detected in the kernel.\n"));
 
 	if (init_mounts_scan(mntcnt, mntpoints, MS_XFS_DISABLED | MS_LOCALONLY) < 0)
@@ -358,7 +384,7 @@ int main(int argc, char **argv)
 	while ((mnt = get_next_mount())) {
 		if (nfs_fstype(mnt->mnt_type)) {
 			if (!(flags & FL_ALL))
-				fprintf(stderr, "%s: Quota cannot be turned on on NFS filesystem\n", mnt->mnt_fsname);
+				errstr(_("%s: Quota cannot be turned on on NFS filesystem\n"), mnt->mnt_fsname);
 			continue;
 		}
 
