@@ -5,6 +5,8 @@
  *	Jan Kara <jack@suse.cz> - sponsored by SuSE CR
  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
@@ -34,9 +36,16 @@
 
 #define min(x,y) (((x) < (y)) ? (x) : (y))
 
+#define QFMT_NAMES 5
+
 static char extensions[MAXQUOTAS + 2][20] = INITQFNAMES;
 static char *basenames[] = INITQFBASENAMES;
-static char *fmtnames[] = INITQFMTNAMES;
+static char *fmtnames[] = { "vfsold",
+			    "vfsv0",
+			    "vfsv1",
+			    "rpc",
+			    "xfs",
+};
 
 /*
  *	Check for various kinds of NFS filesystem
@@ -48,37 +57,12 @@ int nfs_fstype(char *type)
 }
 
 /*
- *	Check whether give filesystem type is supported
+ *	Check whether filesystem has hidden quota files which is handles
+ *	as metadata (and thus always tracks usage).
  */
-
-static int correct_fstype(char *type)
+int meta_qf_fstype(char *type)
 {
-	char *mtype = sstrdup(type), *next;
-
-	type = mtype;
-	do {
-		next = strchr(type, ',');
-		if (next)
-			*next = 0;
-		if (!strcmp(type, MNTTYPE_EXT2) ||
-		    !strcmp(type, MNTTYPE_EXT3) ||
-		    !strcmp(type, MNTTYPE_EXT4) ||
-		    !strcmp(type, MNTTYPE_JFS) ||
-		    !strcmp(type, MNTTYPE_MINIX) ||
-		    !strcmp(type, MNTTYPE_UFS) ||
-		    !strcmp(type, MNTTYPE_UDF) ||
-		    !strcmp(type, MNTTYPE_REISER) ||
-		    !strcmp(type, MNTTYPE_XFS) ||
-		    !strcmp(type, MNTTYPE_NFS) ||
-		    !strcmp(type, MNTTYPE_NFS4) ||
-		    !strcmp(type, MNTTYPE_MPFS)) {
-			free(mtype);
-			return 1;
-		}
-		type = next+1;	
-	} while (next);
-	free(mtype);
-	return 0;
+	return !strcmp(type, MNTTYPE_OCFS2);
 }
 
 /*
@@ -235,12 +219,13 @@ int name2fmt(char *str)
 {
 	int fmt;
 
-	for (fmt = 0; fmt < QUOTAFORMATS; fmt++)
+	for (fmt = 0; fmt < QFMT_NAMES; fmt++)
 		if (!strcmp(str, fmtnames[fmt]))
 			return fmt;
 	errstr(_("Unknown quota format: %s\nSupported formats are:\n\
   vfsold - original quota format\n\
-  vfsv0 - new quota format\n\
+  vfsv0 - standard quota format\n\
+  vfsv1 - quota format with 64-bit limits\n\
   rpc - use RPC calls\n\
   xfs - XFS quota format\n"), str);
 	return QF_ERROR;
@@ -251,22 +236,23 @@ int name2fmt(char *str)
  */
 char *fmt2name(int fmt)
 {
-
-	if (fmt < 0)
-		return _("Unknown format");
 	return fmtnames[fmt];
 }
 
 /*
  *	Convert kernel to utility quota format number
  */
-int kern2utilfmt(int fmt)
+int kern2utilfmt(int kernfmt)
 {
-	switch (fmt) {
+	switch (kernfmt) {
 		case QFMT_VFS_OLD:
 			return QF_VFSOLD;
 		case QFMT_VFS_V0:
 			return QF_VFSV0;
+		case QFMT_VFS_V1:
+			return QF_VFSV1;
+		case QFMT_OCFS2:
+			return QF_META;
 	}
 	return -1;
 }
@@ -281,6 +267,8 @@ int util2kernfmt(int fmt)
 			return QFMT_VFS_OLD;
 		case QF_VFSV0:
 			return QFMT_VFS_V0;
+		case QF_VFSV1:
+			return QFMT_VFS_V1;
 	}
 	return -1;
 }
@@ -366,13 +354,16 @@ void space2str(qsize_t space, char *buf, int format)
 	char suffix[8] = " MGT";
 
 	space = qb2kb(space);
-	if (format)
+	if (format) {
 		for (i = 3; i > 0; i--)
 			if (space >= (1LL << (QUOTABLOCK_BITS*i))*100) {
-				sprintf(buf, "%Lu%c", (unsigned long long)(space+(1 << (QUOTABLOCK_BITS*i))-1) >> (QUOTABLOCK_BITS*i), suffix[i]);
+				sprintf(buf, "%llu%c", (unsigned long long)(space+(1 << (QUOTABLOCK_BITS*i))-1) >> (QUOTABLOCK_BITS*i), suffix[i]);
 				return;
 			}
-	sprintf(buf, "%Lu", (unsigned long long)space);
+		sprintf(buf, "%lluK", (unsigned long long)space);
+		return;
+	}
+	sprintf(buf, "%llu", (unsigned long long)space);
 }
 
 /*
@@ -387,10 +378,10 @@ void number2str(unsigned long long num, char *buf, int format)
 	if (format)
 		for (i = 4, div = 1000000000000LL; i > 0; i--, div /= 1000)
 			if (num >= 100*div) {
-				sprintf(buf, "%Lu%c", (num+div-1) / div, suffix[i]);
+				sprintf(buf, "%llu%c", (num+div-1) / div, suffix[i]);
 				return;
 			}
-	sprintf(buf, "%Lu", num);
+	sprintf(buf, "%llu", num);
 }
 
 /*
@@ -465,10 +456,11 @@ char *hasmntoptarg(struct mntent *mnt, char *opt)
  */
 int hasquota(struct mntent *mnt, int type, int flags)
 {
-	if (!correct_fstype(mnt->mnt_type) || hasmntopt(mnt, MNTOPT_NOQUOTA))
+	if (hasmntopt(mnt, MNTOPT_NOQUOTA))
 		return 0;
 	
-	if (!strcmp(mnt->mnt_type, MNTTYPE_XFS))
+	if (!strcmp(mnt->mnt_type, MNTTYPE_GFS2) ||
+	    !strcmp(mnt->mnt_type, MNTTYPE_XFS))
 		return hasxfsquota(mnt, type, flags);
 	if (nfs_fstype(mnt->mnt_type))	/* NFS always has quota or better there is no good way how to detect it */
 		return 1;
@@ -495,81 +487,85 @@ static int check_fmtfile_ok(char *name, int type, int fmt, int flags)
 				errstr(_("Cannot stat quota file %s: %s\n"), name, strerror(errno));
 			return 0;
 		}
-		return 1;
-	}
-	else {
+	} 
+	if (flags & NF_FORMAT) {
 		int fd, ret = 0;
 
 		if ((fd = open(name, O_RDONLY)) >= 0) {
-			if (fmt == QF_VFSV0)
-				ret = quotafile_ops_2.check_file(fd, type);
+			if (is_tree_qfmt(fmt))
+				ret = quotafile_ops_2.check_file(fd, type, fmt);
 			else
-				ret = quotafile_ops_1.check_file(fd, type);
+				ret = quotafile_ops_1.check_file(fd, type, fmt);
 			close(fd);
+			if (ret <= 0)
+				return 0;
 		}
-		else if (errno != ENOENT && errno != EPERM)
+		else if (errno != ENOENT && errno != EPERM) {
 			errstr(_("Cannot open quotafile %s: %s\n"), name, strerror(errno));
-		return ret;
+			return 0;
+		}
 	}
+	return 1;
 }
 
 /*
- *	Get quotafile name for given entry. Return format and quota file name.
+ *	Get quotafile name for given entry. Return 0 in case format check succeeded,
+ * 	otherwise return -1.
  *	Note that formats without quotafile *must* be detected prior to calling this function
  */
 int get_qf_name(struct mntent *mnt, int type, int fmt, int flags, char **filename)
 {
 	char *option, *pathname, has_quota_file_definition = 0;
-	char qfullname[PATH_MAX] = "";
+	char qfullname[PATH_MAX];
 
+	qfullname[0] = 0;
 	if (type == USRQUOTA && (option = hasmntopt(mnt, MNTOPT_USRQUOTA))) {
 		if (*(pathname = option + strlen(MNTOPT_USRQUOTA)) == '=')
 			has_quota_file_definition = 1;
 	}
 	else if (type == USRQUOTA && (option = hasmntoptarg(mnt, MNTOPT_USRJQUOTA))) {
-		pathname = option-1;
+		pathname = option;
 		has_quota_file_definition = 1;
 		sstrncpy(qfullname, mnt->mnt_dir, sizeof(qfullname));
 		sstrncat(qfullname, "/", sizeof(qfullname));
 	}
 	else if (type == GRPQUOTA && (option = hasmntopt(mnt, MNTOPT_GRPQUOTA))) {
-		if (*(pathname = option + strlen(MNTOPT_GRPQUOTA)) == '=')
+		pathname = option + strlen(MNTOPT_GRPQUOTA);
+		if (*pathname == '=') {
 			has_quota_file_definition = 1;
+			pathname++;
+		}
 	}
 	else if (type == GRPQUOTA && (option = hasmntoptarg(mnt, MNTOPT_GRPJQUOTA))) {
-		pathname = option-1;
+		pathname = option;
 		has_quota_file_definition = 1;
 		sstrncpy(qfullname, mnt->mnt_dir, sizeof(qfullname));
 		sstrncat(qfullname, "/", sizeof(qfullname));
 	}
 	else if (type == USRQUOTA && (option = hasmntopt(mnt, MNTOPT_QUOTA))) {
-		if (*(pathname = option + strlen(MNTOPT_QUOTA)) == '=')
+		pathname = option + strlen(MNTOPT_QUOTA);
+		if (*pathname == '=') {
 			has_quota_file_definition = 1;
+			pathname++;
+		}
 	}
 	else
 		return -1;
 
 	if (has_quota_file_definition) {
-		if ((option = strchr(++pathname, ',')))
-			sstrncpy(qfullname+strlen(qfullname), pathname, min((option - pathname + 1), sizeof(qfullname)-strlen(qfullname)));
-		else
+		if ((option = strchr(pathname, ','))) {
+			int tocopy = min(option - pathname + 1,
+					 sizeof(qfullname) - strlen(qfullname));
+			sstrncpy(qfullname + strlen(qfullname), pathname, tocopy);
+		} else
 			sstrncat(qfullname, pathname, sizeof(qfullname));
+	} else {
+		snprintf(qfullname, PATH_MAX, "%s/%s.%s", mnt->mnt_dir,
+			 basenames[fmt], extensions[type]);
 	}
-	if (fmt & (1 << QF_VFSV0)) {
-		if (!has_quota_file_definition)
-			snprintf(qfullname, PATH_MAX, "%s/%s.%s", mnt->mnt_dir, basenames[QF_VFSV0], extensions[type]);
-		if (check_fmtfile_ok(qfullname, type, QF_VFSV0, flags)) {
-			*filename = sstrdup(qfullname);
-			return QF_VFSV0;
-		}
-	}
-	if (fmt & (1 << QF_VFSOLD)) {
-		if (!has_quota_file_definition)
-			snprintf(qfullname, PATH_MAX, "%s/%s.%s", mnt->mnt_dir, basenames[QF_VFSOLD], extensions[type]);
-		if (check_fmtfile_ok(qfullname, type, QF_VFSOLD, flags)) {
-			*filename = sstrdup(qfullname);
-			return QF_VFSOLD;
-		}
+	if (check_fmtfile_ok(qfullname, type, fmt, flags)) {
+		*filename = sstrdup(qfullname);
+		return 0;
 	}
 	return -1;
 }
@@ -593,10 +589,18 @@ struct quota_handle **create_handle_list(int count, char **mntpoints, int type,
 		hlist_allocated = START_MNT_POINTS;
 	}
 
+	/* If directories are specified, cache all NFS mountpoints */
+	if (count && !(mntflags & MS_LOCALONLY))
+		mntflags |= MS_NFS_ALL;
+
 	if (init_mounts_scan(count, mntpoints, mntflags) < 0)
 		die(2, _("Cannot initialize mountpoint scan.\n"));
 	while ((mnt = get_next_mount())) {
-		if (!nfs_fstype(mnt->mnt_type)) {	/* No NFS? */
+#ifndef RPC
+		if (nfs_fstype(mnt->mnt_type))
+			continue;
+#endif
+		if (fmt == -1 || count) {
 add_entry:
 			if (gotmnt+1 >= hlist_allocated) {
 				hlist_allocated += START_MNT_POINTS;
@@ -606,10 +610,24 @@ add_entry:
 				continue;
 			gotmnt++;
 		}
-		else if (fmt == -1 || fmt == QF_RPC) {	/* Use NFS? */
-#ifdef RPC
-			goto add_entry;
-#endif
+		else {
+			switch (fmt) {
+			case QF_RPC:
+				if (nfs_fstype(mnt->mnt_type))
+					goto add_entry;
+				break;
+			case QF_XFS:
+				if (!strcmp(mnt->mnt_type, MNTTYPE_XFS) ||
+				    !strcmp(mnt->mnt_type, MNTTYPE_GFS2))
+					goto add_entry;
+				break;
+			default:
+				if (strcmp(mnt->mnt_type, MNTTYPE_XFS) &&
+				    strcmp(mnt->mnt_type, MNTTYPE_GFS2) &&
+				    !nfs_fstype(mnt->mnt_type))
+					goto add_entry;
+				break;
+			}
 		}
 	}
 	end_mounts_scan();
@@ -668,7 +686,9 @@ int devcmp_handles(struct quota_handle *a, struct quota_handle *b)
  *	Check kernel quota version
  */
 
-int kernel_iface, kernel_formats;	/* Formats supported by kernel */
+int kernel_iface;	/* Kernel interface type */
+static int kernel_qfmt_num;	/* Number of different supported formats */
+static int kernel_qfmt[QUOTAFORMATS]; /* Formats supported by kernel */
 
 #ifndef FS_DQSTATS
 #define FS_DQSTATS 16
@@ -691,22 +711,28 @@ void init_kernel_interface(void)
 	if (sigaction(SIGSEGV, &sig, &oldsig) < 0)
 		die(2, _("Cannot set signal handler: %s\n"), strerror(errno));
 
-	kernel_formats = 0;
+	kernel_qfmt_num = 0;
 	if (!stat("/proc/fs/xfs/stat", &st))
-		kernel_formats |= (1 << QF_XFS);
-	else
-		if (!quotactl(QCMD(Q_XGETQSTAT, 0), NULL, 0, NULL) || (errno != EINVAL && errno != ENOSYS))
-			kernel_formats |= (1 << QF_XFS);
+		kernel_qfmt[kernel_qfmt_num++] = QF_XFS;
+	else {
+		fs_quota_stat_t dummy;
+
+		if (!quotactl(QCMD(Q_XGETQSTAT, 0), "/dev/root", 0, (void *)&dummy) || (errno != EINVAL && errno != ENOSYS))
+			kernel_qfmt[kernel_qfmt_num++] = QF_XFS;
+	}
 	/* Detect new kernel interface; Assume generic interface unless we can prove there is not one... */
 	if (!stat("/proc/sys/fs/quota", &st) || errno != ENOENT) {
 		kernel_iface = IFACE_GENERIC;
-		kernel_formats |= (1 << QF_VFSOLD) | (1 << QF_VFSV0);
+		kernel_qfmt[kernel_qfmt_num++] = QF_META;
+		kernel_qfmt[kernel_qfmt_num++] = QF_VFSOLD;
+		kernel_qfmt[kernel_qfmt_num++] = QF_VFSV0;
+		kernel_qfmt[kernel_qfmt_num++] = QF_VFSV1;
 	}
 	else {
 		struct v2_dqstats v2_stats;
 
 		if (quotactl(QCMD(Q_V2_GETSTATS, 0), NULL, 0, (void *)&v2_stats) >= 0) {
-			kernel_formats |= (1 << QF_VFSV0);
+			kernel_qfmt[kernel_qfmt_num++] = QF_VFSV0;
 			kernel_iface = IFACE_VFSV0;
 		}
 		else if (errno != ENOSYS && errno != ENOTSUP) {
@@ -726,11 +752,11 @@ void init_kernel_interface(void)
 			 * On a 2.4.x 		we expect 0, ENOENT
 			 * On a 2.4.x-ac	we wont get here */
 			if (err_stat == 0 && err_quota == EINVAL) {
-				kernel_formats |= (1 << QF_VFSV0);
+				kernel_qfmt[kernel_qfmt_num++] = QF_VFSV0;
 				kernel_iface = IFACE_VFSV0;
 			}
 			else {
-				kernel_formats |= (1 << QF_VFSOLD);
+				kernel_qfmt[kernel_qfmt_num++] = QF_VFSOLD;
 				kernel_iface = IFACE_VFSOLD;
 			}
 		}
@@ -739,6 +765,20 @@ void init_kernel_interface(void)
 		die(2, _("Cannot reset signal handler: %s\n"), strerror(errno));
 }
 
+/* Return whether kernel is able to handle given format */
+int kern_qfmt_supp(int fmt)
+{
+	int i;
+
+	if (fmt == -1)
+		return kernel_qfmt_num > 0;
+
+	for (i = 0; i < kernel_qfmt_num; i++)
+		if (fmt == kernel_qfmt[i])
+			return 1;
+	return 0;
+}
+
 /* Check whether old quota is turned on on given device */
 static int v1_kern_quota_on(const char *dev, int type)
 {
@@ -784,19 +824,23 @@ int kern_quota_on(const char *dev, int type, int fmt)
 	if (kernel_iface == IFACE_GENERIC) {
 		int actfmt;
 
-		if (quotactl(QCMD(Q_GETFMT, type), dev, 0, (void *)&actfmt) < 0)
-			return -1;
-		actfmt = kern2utilfmt(actfmt);
-		if (actfmt >= 0 && (fmt == -1 || (1 << actfmt) & fmt))
-			return actfmt;
-		return -1;
+		if (quotactl(QCMD(Q_GETFMT, type), dev, 0,
+			     (void *)&actfmt) >= 0) {
+			actfmt = kern2utilfmt(actfmt);
+			if (actfmt >= 0)
+				return actfmt;
+		}
+	} else {
+		if ((fmt == -1 || fmt == QF_VFSV0) &&
+		    v2_kern_quota_on(dev, type))	/* VFSv0 quota format */
+			return QF_VFSV0;
+		if ((fmt == -1 || fmt == QF_VFSOLD) &&
+		    v1_kern_quota_on(dev, type))	/* Old quota format */
+			return QF_VFSOLD;
 	}
-	if ((fmt & (1 << QF_VFSV0)) && v2_kern_quota_on(dev, type))	/* New quota format */
-		return QF_VFSV0;
-	if ((fmt & (1 << QF_XFS)) && xfs_kern_quota_on(dev, type))	/* XFS quota format */
+	if ((fmt == -1 || fmt == QF_XFS) &&
+	    xfs_kern_quota_on(dev, type))	/* XFS quota format */
 		return QF_XFS;
-	if ((fmt & (1 << QF_VFSOLD)) && v1_kern_quota_on(dev, type))	/* Old quota format */
-		return QF_VFSOLD;
 	return -1;
 }
 
@@ -843,17 +887,20 @@ static int cache_mnt_table(int flags)
 	int autofsdircnt = 0;
 	char autofsdir[AUTOFS_DIR_MAX][PATH_MAX];
 
-	if (!(mntf = setmntent(_PATH_MOUNTED, "r"))) {
-		if (errno != ENOENT) {
-			errstr(_("Cannot open %s: %s\n"), _PATH_MOUNTED, strerror(errno));
-			return -1;
-		}
-		else	/* Fallback on fstab when mtab not available */
-			if (!(mntf = setmntent(_PATH_MNTTAB, "r"))) {
-				errstr(_("Cannot open %s: %s\n"), _PATH_MNTTAB, strerror(errno));
-				return -1;
-			}
+#ifdef ALT_MTAB
+	mntf = setmntent(ALT_MTAB, "r");
+	if (mntf)
+		goto alloc;
+#endif
+	mntf = setmntent(_PATH_MOUNTED, "r");
+	if (mntf)
+		goto alloc;
+	/* Fallback to fstab when mtab not available */
+	if (!(mntf = setmntent(_PATH_MNTTAB, "r"))) {
+		errstr(_("Cannot open any file with mount points.\n"));
+		return -1;
 	}
+alloc:
 	mnt_entries = smalloc(sizeof(struct mount_entry) * ALLOC_ENTRIES_NUM);
 	mnt_entries_cnt = 0;
 	allocated += ALLOC_ENTRIES_NUM;
@@ -974,9 +1021,9 @@ static int cache_mnt_table(int flags)
 				continue;
 			}
 			if (nfs_fstype(mnt->mnt_type)) {
+				/* For network filesystems we must get device from root */
+				dev = st.st_dev;
 				if (!(flags & MS_NFS_ALL)) {
-					/* For network filesystems we must get device from root */
-					dev = st.st_dev;
 					for (i = 0; i < mnt_entries_cnt && mnt_entries[i].me_dev != dev; i++);
 				}
 				else	/* Always behave as if the device was unique */
