@@ -6,6 +6,8 @@
  *	Jan Kara <jack@suse.cz> - Sponsored by SuSE CZ
  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -36,11 +38,11 @@
 #define FL_NOAUTOFS 256	/* Ignore autofs mountpoints */
 #define FL_RAWGRACE 512	/* Print grace times in seconds since epoch */
 
-int flags, fmt = -1;
-char **mnt;
-int mntcnt;
-int cached_dquots;
-struct dquot dquot_cache[MAX_CACHE_DQUOTS];
+static int flags, fmt = -1;
+static char **mnt;
+static int mntcnt;
+static int cached_dquots;
+static struct dquot dquot_cache[MAX_CACHE_DQUOTS];
 char *progname;
 
 static void usage(void)
@@ -50,7 +52,7 @@ static void usage(void)
 -u, --user                  display information about users\n\
 -g, --group                 display information about groups\n\
 -s, --human-readable        show numbers in human friendly units (MB, GB, ...)\n\
--t, --truncate-names        truncate names to 8 characters\n\
+-t, --truncate-names        truncate names to 9 characters\n\
 -p, --raw-grace             print grace time in seconds since epoch\n\
 -n, --no-names              do not translate uid/gid to name\n\
 -i, --no-autofs             avoid autofs mountpoints\n\
@@ -158,7 +160,7 @@ static void parse_options(int argcnt, char **argstr)
 }
 
 /* Are we over soft or hard limit? */
-static char overlim(uint usage, uint softlim, uint hardlim)
+static char overlim(qsize_t usage, qsize_t softlim, qsize_t hardlim)
 {
 	if ((usage > softlim && softlim) || (usage > hardlim && hardlim))
 		return '+';
@@ -181,7 +183,7 @@ static void print(struct dquot *dquot, char *name)
 		pname[PRINTNAMELEN] = 0;
 	if (entry->dqb_bsoftlimit && toqb(entry->dqb_curspace) >= entry->dqb_bsoftlimit)
 		if (flags & FL_RAWGRACE)
-			sprintf(time, "%Lu", (unsigned long long)entry->dqb_btime);
+			sprintf(time, "%llu", (unsigned long long)entry->dqb_btime);
 		else
 			difftime2str(entry->dqb_btime, time);
 	else
@@ -198,7 +200,7 @@ static void print(struct dquot *dquot, char *name)
 	       numbuf[0], numbuf[1], numbuf[2], time);
 	if (entry->dqb_isoftlimit && entry->dqb_curinodes >= entry->dqb_isoftlimit)
 		if (flags & FL_RAWGRACE)
-			sprintf(time, "%Lu", (unsigned long long)entry->dqb_itime);
+			sprintf(time, "%llu", (unsigned long long)entry->dqb_itime);
 		else
 			difftime2str(entry->dqb_itime, time);
 	else
@@ -284,12 +286,18 @@ static int output(struct dquot *dquot, char *name)
 static void report_it(struct quota_handle *h, int type)
 {
 	char bgbuf[MAXTIMELEN], igbuf[MAXTIMELEN];
+	char *spacehdr;
+
+	if (flags & FL_SHORTNUMS)
+		spacehdr = _("Space");
+	else
+		spacehdr = _("Block");
 
 	printf(_("*** Report for %s quotas on device %s\n"), type2name(type), h->qh_quotadev);
 	time2str(h->qh_info.dqi_bgrace, bgbuf, TF_ROUND);
 	time2str(h->qh_info.dqi_igrace, igbuf, TF_ROUND);
 	printf(_("Block grace time: %s; Inode grace time: %s\n"), bgbuf, igbuf);
-	printf(_("                        Block limits                File limits\n"));
+	printf(_("                        %s limits                File limits\n"), spacehdr);
 	printf(_("%-9s       used    soft    hard  grace    used  soft  hard  grace\n"), (type == USRQUOTA)?_("User"):_("Group"));
 	printf("----------------------------------------------------------------------\n");
 
@@ -309,9 +317,9 @@ static void report(int type)
 	int i;
 
 	if (flags & FL_ALL)
-		handles = create_handle_list(0, NULL, type, fmt, IOI_READONLY | IOI_OPENFILE, MS_LOCALONLY | (flags & FL_NOAUTOFS ? MS_NO_AUTOFS : 0));
+		handles = create_handle_list(0, NULL, type, fmt, IOI_READONLY | IOI_INITSCAN, MS_LOCALONLY | (flags & FL_NOAUTOFS ? MS_NO_AUTOFS : 0));
 	else
-		handles = create_handle_list(mntcnt, mnt, type, fmt, IOI_READONLY | IOI_OPENFILE, MS_LOCALONLY | (flags & FL_NOAUTOFS ? MS_NO_AUTOFS : 0));
+		handles = create_handle_list(mntcnt, mnt, type, fmt, IOI_READONLY | IOI_INITSCAN, MS_LOCALONLY | (flags & FL_NOAUTOFS ? MS_NO_AUTOFS : 0));
 	for (i = 0; handles[i]; i++)
 		report_it(handles[i], type);
 	dispose_handle_list(handles);
