@@ -9,8 +9,6 @@
  *
  *          This part does the rpc-communication with the rquotad.
  *
- * Version: $Id: rquota_client.c,v 1.9 2005/03/31 11:48:02 jkar8572 Exp $
- *
  * Author:  Marco van Wieringen <mvw@planets.elm.net>
  *
  *          This program is free software; you can redistribute it and/or
@@ -18,6 +16,9 @@
  *          as published by the Free Software Foundation; either version
  *          2 of the License, or (at your option) any later version.
  */
+
+#include "config.h"
+
 #include <rpc/rpc.h>
 #include <sys/types.h>
 #include <sys/param.h>
@@ -45,8 +46,8 @@ static inline void clinet2utildqblk(struct util_dqblk *u, struct rquota *n)
 	time_t now;
 	
 	/* Copy the quota */
-	u->dqb_bhardlimit = n->rq_bhardlimit;
-	u->dqb_bsoftlimit = n->rq_bsoftlimit;
+	u->dqb_bhardlimit = toqb(((qsize_t)n->rq_bhardlimit) * n->rq_bsize);
+	u->dqb_bsoftlimit = toqb(((qsize_t)n->rq_bsoftlimit) * n->rq_bsize);
 	u->dqb_ihardlimit = n->rq_fhardlimit;
 	u->dqb_isoftlimit = n->rq_fsoftlimit;
 	u->dqb_curinodes = n->rq_curfiles;
@@ -60,22 +61,6 @@ static inline void clinet2utildqblk(struct util_dqblk *u, struct rquota *n)
 		u->dqb_itime = n->rq_ftimeleft + now;
 	else
 		u->dqb_itime = 0;
-	/* Convert from remote block size */
-	if (n->rq_bsize != RPC_DQBLK_SIZE) {
-		int conversion_unit;
-
-		conversion_unit = n->rq_bsize >> RPC_DQBLK_SIZE_BITS;
-		if (conversion_unit == 0) {
-			conversion_unit = RPC_DQBLK_SIZE / n->rq_bsize;
-
-			u->dqb_bhardlimit /= conversion_unit;
-			u->dqb_bsoftlimit /= conversion_unit;
-		}
-		else {
-			u->dqb_bhardlimit *= conversion_unit;
-			u->dqb_bsoftlimit *= conversion_unit;
-		}
-	}
 }
 
 /* Convert utils format of quotas to network one */
@@ -101,7 +86,7 @@ static inline void cliutil2netdqblk(struct sq_dqblk *n, struct util_dqblk *u)
 }
 
 /* Write appropriate error message */
-int rquota_err(int stat)
+static int rquota_err(int stat)
 {
 	switch (stat) {
 		case -1:
@@ -119,6 +104,35 @@ int rquota_err(int stat)
 	}
 }
 
+static int split_nfs_mount(char *devname, char **host, char **path)
+{
+	char *pathname;
+
+	/* NFS server name contained in brackets? */
+	if (*devname == '[') {
+		*host = devname + 1;
+		pathname = strchr(devname, ']');
+		if (!pathname || pathname[1] != ':')
+			return 0;
+		/* Autofs? */
+		if (pathname[2] == '(')
+			return 0;
+		*pathname = 0;
+		*path = pathname + 2;
+		return 1;
+	}
+	*host = devname;
+	pathname = strchr(devname, ':');
+	if (!pathname)
+		return 0;
+	/* Autofs? */
+	if (pathname[1] == '(')
+		return 0;
+	*pathname = 0;
+	*path = pathname + 1;
+	return 1;
+}
+
 /*
  * Collect the requested quota information from a remote host.
  */
@@ -143,18 +157,19 @@ int rpc_rquota_get(struct dquot *dquot)
 	 */
 	fsname_tmp = (char *)smalloc(strlen(dquot->dq_h->qh_quotadev) + 1);
 	strcpy(fsname_tmp, dquot->dq_h->qh_quotadev);
-	host = fsname_tmp;
-
-	/*
-	 * Strip off pathname on nfs mounted dir. Ignore entries of any
-	 * automounter.
-	 */
-	if ((pathname = strchr(fsname_tmp, ':')) == (char *)0 || *(pathname + 1) == '(') {
+	if (!split_nfs_mount(fsname_tmp, &host, &pathname)) {
 		free(fsname_tmp);
 		return -ENOENT;
 	}
 
 	*pathname++ = '\0';
+	/* For NFSv4, we send the filesystem path without initial /. Server prepends proper
+	 * NFS pseudoroot automatically and uses this for detection of NFSv4 mounts. */
+	if ((dquot->dq_h->qh_io_flags & IOFL_NFS_MIXED_PATHS) &&
+	    !strcmp(dquot->dq_h->qh_fstype, MNTTYPE_NFS4)) {
+		while (*pathname == '/')
+			pathname++;
+	}
 
 	/*
 	 * First try EXT_RQUOTAPROG (Extended (LINUX) RPC quota program)
@@ -255,16 +270,18 @@ int rpc_rquota_set(int qcmd, struct dquot *dquot)
 	 */
 	fsname_tmp = (char *)smalloc(strlen(dquot->dq_h->qh_quotadev) + 1);
 	strcpy(fsname_tmp, dquot->dq_h->qh_quotadev);
-	host = fsname_tmp;
-
-	/*
-	 * Strip off pathname on nfs mounted dir. Ignore entries of any
-	 * automounter.
-	 */
-	if ((pathname = strchr(fsname_tmp, ':')) == (char *)0 || *(pathname + 1) == '(')
+	if (!split_nfs_mount(fsname_tmp, &host, &pathname)) {
+		free(fsname_tmp);
 		return -ENOENT;
+	}
 
-	*pathname++ = '\0';
+	/* For NFSv4, we send the filesystem path without initial /. Server prepends proper
+	 * NFS pseudoroot automatically and uses this for detection of NFSv4 mounts. */
+	if ((dquot->dq_h->qh_io_flags & IOFL_NFS_MIXED_PATHS) &&
+	    !strcmp(dquot->dq_h->qh_fstype, MNTTYPE_NFS4)) {
+		while (*pathname == '/')
+			pathname++;
+	}
 
 	/*
 	 * First try EXT_RQUOTAPROG (Extended (LINUX) RPC quota program)
