@@ -12,14 +12,14 @@
  *          changes for new utilities by Jan Kara <jack@suse.cz>
  *          patches by Jani Jaakkola <jjaakkol@cs.helsinki.fi>
  *
- * Version: $Id: rquota_svc.c,v 1.17 2005/06/01 07:20:50 jkar8572 Exp $
- *
  *          This program is free software; you can redistribute it and/or
  *          modify it under the terms of the GNU General Public License as
  *          published by the Free Software Foundation; either version 2 of
  *          the License, or (at your option) any later version.
  */
                                                                                                           
+#include "config.h"
+
 #include <rpc/rpc.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -36,6 +36,8 @@
 #ifdef HOSTS_ACCESS
 #include <tcpd.h>
 #include <netdb.h>
+
+int deny_severity, allow_severity;	/* Needed by some versions of libwrap */
 #endif
 
 #ifdef __STDC__
@@ -84,23 +86,23 @@ static void show_help(void)
 {
 #ifdef RPC_SETQUOTA
 	errstr(_("Usage: %s [options]\nOptions are:\n\
- -h --help         shows this text\n\
- -V --version      shows version information\n\
- -F --foreground   starts the quota service in foreground\n\
- -I --autofs       do not ignore mountpoints mounted by automounter\n\
- -p --port <port>  listen on given port\n\
- -s --no-setquota  disables remote calls to setquota (default)\n\
- -S --setquota     enables remote calls to setquota\n\
- -x --xtab <path>  set an alternative file with NFSD export table\n"), progname);
+ -h --help             shows this text\n\
+ -V --version          shows version information\n\
+ -F --foreground       starts the quota service in foreground\n\
+ -I --autofs           do not ignore mountpoints mounted by automounter\n\
+ -p --port <port>      listen on given port\n\
+ -s --no-setquota      disables remote calls to setquota (default)\n\
+ -S --setquota         enables remote calls to setquota\n\
+ -x --xtab <path>      set an alternative file with NFSD export table\n"), progname);
 
 #else
 	errstr(_("Usage: %s [options]\nOptions are:\n\
- -h --help         shows this text\n\
- -V --version      shows version information\n\
- -F --foreground   starts the quota service in foreground\n\
- -I --autofs       do not ignore mountpoints mounted by automounter\n\
- -p --port <port>  listen on given port\n\
- -x --xtab <path>  set an alternative file with NFSD export table\n"), progname);
+ -h --help             shows this text\n\
+ -V --version          shows version information\n\
+ -F --foreground       starts the quota service in foreground\n\
+ -I --autofs           do not ignore mountpoints mounted by automounter\n\
+ -p --port <port>      listen on given port\n\
+ -x --xtab <path>      set an alternative file with NFSD export table\n"), progname);
 #endif
 }
 
@@ -167,13 +169,12 @@ static void parse_options(int argc, char **argv)
  * good_client checks if an quota client should be allowed to
  * execute the requested rpc call.
  */
-int good_client(struct sockaddr_in *addr, ulong rq_proc)
+static int good_client(struct sockaddr_in *addr, ulong rq_proc)
 {
 #ifdef HOSTS_ACCESS
-	struct hostent *h;
-	char *name, **ad;
+	struct request_info req;
 #endif
-	const char *remote=inet_ntoa(addr->sin_addr);
+	char *remote = inet_ntoa(addr->sin_addr);
 
 	if (rq_proc==RQUOTAPROC_SETQUOTA ||
 	     rq_proc==RQUOTAPROC_SETACTIVEQUOTA) {
@@ -197,31 +198,10 @@ int good_client(struct sockaddr_in *addr, ulong rq_proc)
 	/* NOTE: we could use different servicename for setquota calls to
 	 * allow only some hosts to call setquota. */
 
-	/* Check IP address */
-	if (hosts_ctl("rquotad", "", remote, ""))
+	request_init(&req, RQ_DAEMON, "rquotad", RQ_CLIENT_SIN, addr, 0);
+	sock_methods(&req);
+	if (hosts_access(&req))
 		return 1;
-	/* Get address */
-	if (!(h = gethostbyaddr((const char *)&(addr->sin_addr), sizeof(addr->sin_addr), AF_INET)))
-		goto denied;
-	if (!(name = alloca(strlen(h->h_name)+1)))
-		goto denied;
-	strcpy(name, h->h_name);
-	/* Try to resolve it back */
-	if (!(h = gethostbyname(name)))
-		goto denied;
-	for (ad = h->h_addr_list; *ad; ad++)
-		if (!memcmp(*ad, &(addr->sin_addr), h->h_length))
-			break;
-	if (!*ad)	/* Our address not found? */
-		goto denied;	
-	/* Check host name */
-	if (hosts_ctl("rquotad", "", h->h_name, ""))
-		return 1;
-	/* Check aliases */
-	for (ad = h->h_aliases; *ad; ad++)
-		if (hosts_ctl("rquotad", "", *ad, ""))
-			return 1;
-denied:
 	errstr(_("Denied access to host %s\n"), remote);
 	return 0;
 #else
