@@ -1,3 +1,5 @@
+#include "config.h"
+
 #include <sys/types.h>
 #include <sys/quota.h>
 #include <errno.h>
@@ -5,7 +7,7 @@
 
 #include "pot.h"
 
-int copy_user_quota_limits(const char *block_device, uid_t from, uid_t to)
+static int copy_user_quota_limits(const char *block_device, uid_t from, uid_t to)
 {
 	struct dqblk dq;
 
@@ -28,7 +30,7 @@ int copy_user_quota_limits(const char *block_device, uid_t from, uid_t to)
 	}
 }
 
-int copy_group_quota_limits(const char *block_device, gid_t from, gid_t to)
+static int copy_group_quota_limits(const char *block_device, gid_t from, gid_t to)
 {
 	struct dqblk dq;
 
