@@ -4,6 +4,9 @@
  *
  *	Jan Kara <jack@suse.cz> - sponsored by SuSE CR
  */
+
+#include "config.h"
+
 #include <rpc/rpc.h>
 #include <sys/types.h>
 #include <errno.h>
@@ -34,12 +37,12 @@
 #define FL_NO_MIXED_PATHS 512
 #define FL_CONTINUE_BATCH 1024
 
-int flags, fmt = -1;
-char **mnt;
+static int flags, fmt = -1;
+static char **mnt;
 char *progname;
-int mntcnt;
-qid_t protoid, id;
-struct util_dqblk toset;
+static int mntcnt;
+static qid_t protoid, id;
+static struct util_dqblk toset;
 
 /* Print usage information */
 static void usage(void)
@@ -78,10 +81,10 @@ static void usage(void)
 }
 
 /* Convert string to number - print errstr message in case of failure */
-static long parse_num(char *str, char *msg)
+static qsize_t parse_unum(char *str, char *msg)
 {
 	char *errch;
-	long ret = strtol(str, &errch, 0);
+	qsize_t ret = strtoull(str, &errch, 0);
 
 	if (*errch) {
 		errstr(_("Bad %s: %s\n"), msg, str);
@@ -107,9 +110,9 @@ static void parse_options(int argcnt, char **argstr)
 	char *protoname = NULL;
 
 #ifdef RPC_SETQUOTA
-	char *opts = "gp:urmVF:taTbc";
+	char *opts = "ghp:urmVF:taTbc";
 #else
-	char *opts = "gp:uVF:taTbc";
+	char *opts = "ghp:uVF:taTbc";
 #endif
 	struct option long_opts[] = {
 		{ "user", 0, NULL, 'u' },
@@ -184,7 +187,7 @@ static void parse_options(int argcnt, char **argstr)
 		usage();
 	}
 	if (flags & FL_PROTO && flags & FL_GRACE) {
-		errstr(_("Prototype user has no sense when editting grace times.\n"));
+		errstr(_("Prototype user has no sense when editing grace times.\n"));
 		usage();
 	}
 	if (flags & FL_INDIVIDUAL_GRACE && flags & FL_GRACE) {
@@ -199,6 +202,10 @@ static void parse_options(int argcnt, char **argstr)
 		errstr(_("Batch mode and prototype user cannot be used together.\n"));
 		usage();
 	}
+	if (flags & FL_RPC && (flags & (FL_GRACE | FL_INDIVIDUAL_GRACE))) {
+		errstr(_("Cannot set grace times over RPC protocol.\n"));
+		usage();
+	}
 	if (flags & FL_GRACE)
 		otherargs = 2;
 	else if (flags & FL_INDIVIDUAL_GRACE)
@@ -219,17 +226,17 @@ static void parse_options(int argcnt, char **argstr)
 	if (!(flags & (FL_GRACE | FL_BATCH))) {
 		id = name2id(argstr[optind++], flag2type(flags), !!(flags & FL_NUMNAMES), NULL);
 		if (!(flags & (FL_GRACE | FL_INDIVIDUAL_GRACE | FL_PROTO))) {
-			toset.dqb_bsoftlimit = parse_num(argstr[optind++], _("block softlimit"));
-			toset.dqb_bhardlimit = parse_num(argstr[optind++], _("block hardlimit"));
-			toset.dqb_isoftlimit = parse_num(argstr[optind++], _("inode softlimit"));
-			toset.dqb_ihardlimit = parse_num(argstr[optind++], _("inode hardlimit"));
+			toset.dqb_bsoftlimit = parse_unum(argstr[optind++], _("block softlimit"));
+			toset.dqb_bhardlimit = parse_unum(argstr[optind++], _("block hardlimit"));
+			toset.dqb_isoftlimit = parse_unum(argstr[optind++], _("inode softlimit"));
+			toset.dqb_ihardlimit = parse_unum(argstr[optind++], _("inode hardlimit"));
 		}
 		else if (flags & FL_PROTO)
 			protoid = name2id(protoname, flag2type(flags), !!(flags & FL_NUMNAMES), NULL);
 	}
 	if (flags & FL_GRACE) {
-		toset.dqb_btime = parse_num(argstr[optind++], _("block grace time"));
-		toset.dqb_itime = parse_num(argstr[optind++], _("inode grace time"));
+		toset.dqb_btime = parse_unum(argstr[optind++], _("block grace time"));
+		toset.dqb_itime = parse_unum(argstr[optind++], _("inode grace time"));
 	}
 	else if (flags & FL_INDIVIDUAL_GRACE) {
 		time_t now;
@@ -240,13 +247,13 @@ static void parse_options(int argcnt, char **argstr)
 			optind++;
 		}
 		else
-			toset.dqb_btime = now + parse_num(argstr[optind++], _("block grace time"));
+			toset.dqb_btime = now + parse_unum(argstr[optind++], _("block grace time"));
 		if (!strcmp(argstr[optind], _("unset"))) {
 			toset.dqb_itime = 0;
 			optind++;
 		}
 		else
-			toset.dqb_itime = now + parse_num(argstr[optind++], _("inode grace time"));
+			toset.dqb_itime = now + parse_unum(argstr[optind++], _("inode grace time"));
 	}
 	if (!(flags & FL_ALL)) {
 		mntcnt = argcnt - optind;
@@ -369,14 +376,19 @@ static int batch_setlimits(struct quota_handle **handles)
 /* Set grace times */
 static int setgraces(struct quota_handle **handles)
 {
-	int i;
+	int i, ret = 0;
 
 	for (i = 0; handles[i]; i++) {
+		if (!handles[i]->qh_ops->write_info) {
+			errstr(_("Setting grace period on %s is not supported.\n"), handles[i]->qh_quotadev);
+			ret = -1;
+			continue;
+		}
 		handles[i]->qh_info.dqi_bgrace = toset.dqb_btime;
 		handles[i]->qh_info.dqi_igrace = toset.dqb_itime;
 		mark_quotafile_info_dirty(handles[i]);
 	}
-	return 0;
+	return ret;
 }
 
 /* Set grace times for individual user */
@@ -387,8 +399,14 @@ static int setindivgraces(struct quota_handle **handles)
 
 	curprivs = getprivs(id, handles, 0);
 	for (q = curprivs; q; q = q->dq_next) {
-		q->dq_dqb.dqb_btime = toset.dqb_btime;
-		q->dq_dqb.dqb_itime = toset.dqb_itime;
+		if (q->dq_dqb.dqb_bsoftlimit && toqb(q->dq_dqb.dqb_curspace) > q->dq_dqb.dqb_bsoftlimit)
+			q->dq_dqb.dqb_btime = toset.dqb_btime;
+		else
+			errstr(_("Not setting block grace time on %s because softlimit is not exceeded.\n"), q->dq_h->qh_quotadev);
+		if (q->dq_dqb.dqb_isoftlimit && q->dq_dqb.dqb_curinodes > q->dq_dqb.dqb_isoftlimit)
+			q->dq_dqb.dqb_itime = toset.dqb_itime;
+		else
+			errstr(_("Not setting inode grace time on %s because softlimit is not exceeded.\n"), q->dq_h->qh_quotadev);
 	}
 	if (putprivs(curprivs, COMMIT_TIMES) == -1) {
 		errstr(_("cannot write times for %s. Maybe kernel does not support such operation?\n"), type2name(flags & FL_USER ? USRQUOTA : GRPQUOTA));
